use32

; =============================================================================
; Syntax:      (reserved token name "error")
; Description: For internal use. Gets called in the case that a syntax error appeared.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 3 bytes)
; =============================================================================
; 2B C0 C3 (3.0.4 Original ASM)
; 29 C0 C3 (FASM Re-Assemble)

db "error", 0, 0, 0
dw FUNC0_END - FUNC0_START

FUNC0_START:
sub    eax,eax
ret
FUNC0_END:

; =============================================================================
; Syntax:      (reserved token name "prolog")
; Description: For internal use. Gets called at the beginning of the compiled channel asm code.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 8 bytes)
; =============================================================================
; 55 56 57 54 8B 7C 24 14 (3.0.4 Original ASM)
; 55 56 57 54 8B 7C 24 14 (FASM Re-Assemble)

db "prolog", 0, 0
dw FUNC1_END - FUNC1_START

FUNC1_START:
push   ebp
push   esi
push   edi
push   esp
mov    edi,DWORD [esp+0x14] 
FUNC1_END:

; =============================================================================
; Syntax:      (reserved token name "epilog")
; Description: For internal use. Gets called at the end of the compiled channel asm code.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; 58 5C 5F 5E 5D C3 (3.0.4 Original ASM)
; 58 5C 5F 5E 5D C3 (FASM Re-Assemble)

db "epilog", 0, 0
dw FUNC2_END - FUNC2_START

FUNC2_START:
pop    eax
pop    esp
pop    edi
pop    esi
pop    ebp
ret 
FUNC2_END:

; =============================================================================
; Syntax:      (reserved token name "number")
; Description: For internal use. ASM-Prefix to every number. The number itself is +4 byte ASM size.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 1+4 bytes)
; =============================================================================
; 68 __ __ __ __ (3.0.4 Original ASM)
; 68 __ __ __ __ (FASM Re-Assemble)

db "number", 0, 0
dw FUNC3_END - FUNC3_START

FUNC3_START:
db 104 ; the next 4 bytes in this instruction will be added by the plugin!
FUNC3_END:

; =============================================================================
; Syntax:      a,b
; Description: Inside a function: Separates two expressions.
;              Outside a function: "Forget everything" and start over again.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 1 byte)
; =============================================================================
; 58 (3.0.4 Original ASM)
; 58 (FASM Re-Assemble)

db ",", 0, 0, 0, 0, 0, 0, 0
dw FUNC4_END - FUNC4_START

FUNC4_START:
pop    eax
FUNC4_END:

; =============================================================================
; Syntax:      a?b:c
; Description: If a then b else c
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 11 bytes)
; =============================================================================
; 58 5B 59 83 F9 00 74 02 8B C3 50 (3.0.4 Original ASM)
; 58 5B 59 83 F9 00 74 02 89 D8 50 (FASM Re-Assemble)

db "?", 0, 0, 0, 0, 0, 0, 0
dw FUNC5_END - FUNC5_START

FUNC5_START:
pop    eax
pop    ebx
pop    ecx
cmp    ecx,0x0
je     FUNC5_START+0xa
mov    eax,ebx
push   eax 
FUNC5_END:

; =============================================================================
; Syntax:      !a
; Description: Logical "not"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 9 bytes)
; =============================================================================
; 58 83 F8 01 1B C0 F7 D8 50 (3.0.4 Original ASM)
; 58 83 F8 01 19 C0 F7 D8 50 (FASM Re-Assemble)

db "!", 0, 0, 0, 0, 0, 0, 0
dw FUNC6_END - FUNC6_START

FUNC6_START:
pop    eax
cmp    eax,0x1
sbb    eax,eax
neg    eax
push   eax 
FUNC6_END:

; =============================================================================
; Syntax:      a&&b
; Description: Logical "and"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 19 bytes)
; =============================================================================
; 58 5B 83 F8 00 B8 00 00 00 00 74 06 83 FB 00 74 01 40 50 (3.0.4 Original ASM)
; 58 5B 83 F8 00 B8 00 00 00 00 74 06 83 FB 00 74 01 40 50 (FASM Re-Assemble)

db "&&", 0, 0, 0, 0, 0, 0
dw FUNC7_END - FUNC7_START

FUNC7_START:
pop    eax
pop    ebx
cmp    eax,0x0
mov    eax,0x0
je     FUNC7_START+0x12
cmp    ebx,0x0
je     FUNC7_START+0x12
inc    eax
push   eax 
FUNC7_END:

; =============================================================================
; Syntax:      a||b
; Description: Logical "or"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 19 bytes)
; =============================================================================
; 58 5B 83 F8 00 B8 00 00 00 00 75 05 83 FB 00 74 01 40 50 (3.0.4 Original ASM)
; 58 5B 83 F8 00 B8 00 00 00 00 75 05 83 FB 00 74 01 40 50 (FASM Re-Assemble)

db "||", 0, 0, 0, 0, 0, 0
dw FUNC8_END - FUNC8_START

FUNC8_START:
pop    eax
pop    ebx
cmp    eax,0x0
mov    eax,0x0
jne    FUNC8_START+0x11
cmp    ebx,0x0
je     FUNC8_START+0x12
inc    eax
push   eax 
FUNC8_END:

; =============================================================================
; Syntax:      ~a
; Description: Bitwise "not"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 58 F7 D0 50 (3.0.4 Original ASM)
; 58 F7 D0 50 (FASM Re-Assemble)

db "~", 0, 0, 0, 0, 0, 0, 0
dw FUNC9_END - FUNC9_START

FUNC9_START:
pop    eax
not    eax
push   eax 
FUNC9_END:

; =============================================================================
; Syntax:      a&b
; Description: Bitwise "and"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 58 5B 23 C3 50 (3.0.4 Original ASM)
; 58 5B 21 D8 50 (FASM Re-Assemble)

db "&", 0, 0, 0, 0, 0, 0, 0
dw FUNC10_END - FUNC10_START

FUNC10_START:
pop    eax
pop    ebx
and    eax,ebx
push   eax 
FUNC10_END:

; =============================================================================
; Syntax:      a|b
; Description: Bitwise "or"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 58 5B 0B C3 50 (3.0.4 Original ASM)
; 58 5B 09 D8 50 (FASM Re-Assemble)

db "|", 0, 0, 0, 0, 0, 0, 0
dw FUNC11_END - FUNC11_START

FUNC11_START:
pop    eax
pop    ebx
or     eax,ebx
push   eax 
FUNC11_END:

; =============================================================================
; Syntax:      a^b
; Description: Bitwise "xor"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 58 5B 33 C3 50 (3.0.4 Original ASM)
; 58 5B 31 D8 50 (FASM Re-Assemble)

db "^", 0, 0, 0, 0, 0, 0, 0
dw FUNC12_END - FUNC12_START

FUNC12_START:
pop    eax
pop    ebx
xor    eax,ebx
push   eax 
FUNC12_END:

; =============================================================================
; Syntax:      a==b
; Description: Equal?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 94 C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 94 C0 50 (FASM Re-Assemble)

db "==", 0, 0, 0, 0, 0, 0
dw FUNC13_END - FUNC13_START

FUNC13_START:
pop    eax
pop    ebx
cmp    eax,ebx
mov    eax,0x0
sete   al
push   eax 
FUNC13_END:

; =============================================================================
; Syntax:      a!=b
; Description: Not equal?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 95 C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 95 C0 50 (FASM Re-Assemble)

db "!=", 0, 0, 0, 0, 0, 0
dw FUNC14_END - FUNC14_START

FUNC14_START:
pop    eax
pop    ebx
cmp    eax,ebx
mov    eax,0x0
setne  al
push   eax 
FUNC14_END:

; =============================================================================
; Syntax:      a<b
; Description: a smaller than b?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 9F C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 9F C0 50 (FASM Re-Assemble)

db "<", 0, 0, 0, 0, 0, 0, 0
dw FUNC15_END - FUNC15_START

FUNC15_START:
pop    eax
pop    ebx
cmp    eax,ebx
mov    eax,0x0
setg   al
push   eax 
FUNC15_END:

; =============================================================================
; Syntax:      a<=b
; Description: a smaller or equal to b?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 9D C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 9D C0 50 (FASM Re-Assemble)

db "<=", 0, 0, 0, 0, 0, 0
dw FUNC16_END - FUNC16_START

FUNC16_START:
pop    eax
pop    ebx
cmp    eax,ebx
mov    eax,0x0
setge  al
push   eax 
FUNC16_END:

; =============================================================================
; Syntax:      a>b
; Description: a greater than b?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 9C C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 9C C0 50 (FASM Re-Assemble)

db ">", 0, 0, 0, 0, 0, 0, 0
dw FUNC17_END - FUNC17_START

FUNC17_START:
pop    eax
pop    ebx
cmp    eax,ebx
mov    eax,0x0
setl   al
push   eax 
FUNC17_END:

; =============================================================================
; Syntax:      a>=b
; Description: a greater or equal b?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 9E C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 9E C0 50 (FASM Re-Assemble)

db ">=", 0, 0, 0, 0, 0, 0
dw FUNC18_END - FUNC18_START

FUNC18_START:
pop    eax
pop    ebx
cmp    eax,ebx
mov    eax,0x0
setle  al
push   eax 
FUNC18_END:

; =============================================================================
; Syntax:      a<<b
; Description: Left shift
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 59 58 D3 E0 50 (3.0.4 Original ASM)
; 59 58 D3 E0 50 (FASM Re-Assemble)

db "<<", 0, 0, 0, 0, 0, 0
dw FUNC19_END - FUNC19_START

FUNC19_START:
pop    ecx
pop    eax
shl    eax,cl
push   eax 
FUNC19_END:

; =============================================================================
; Syntax:      a>>b
; Description: Right shift
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 59 58 D3 E8 50 (3.0.4 Original ASM)
; 59 58 D3 E8 50 (FASM Re-Assemble)

db ">>", 0, 0, 0, 0, 0, 0
dw FUNC20_END - FUNC20_START

FUNC20_START:
pop    ecx
pop    eax
shr    eax,cl
push   eax 
FUNC20_END:

; =============================================================================
; Syntax:      (reserved token name "negate")
; Description: For internal use only. The real sign is "-a".
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 58 F7 D8 50 (3.0.4 Original ASM)
; 58 F7 D8 50 (FASM Re-Assemble)

db "negate", 0, 0
dw FUNC21_END - FUNC21_START

FUNC21_START:
pop    eax
neg    eax
push   eax 
FUNC21_END:

; =============================================================================
; Syntax:      a+b
; Description: Add
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 58 5B 03 C3 50 (3.0.4 Original ASM)
; 58 5B 01 D8 50 (FASM Re-Assemble)

db "+", 0, 0, 0, 0, 0, 0, 0
dw FUNC22_END - FUNC22_START

FUNC22_START:
pop    eax
pop    ebx
add    eax,ebx
push   eax 
FUNC22_END:

; =============================================================================
; Syntax:      a-b
; Description: Minus
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 5B 58 2B C3 50 (3.0.4 Original ASM)
; 5B 58 29 D8 50 (FASM Re-Assemble)

db "-", 0, 0, 0, 0, 0, 0, 0
dw FUNC23_END - FUNC23_START

FUNC23_START:
pop    ebx
pop    eax
sub    eax,ebx
push   eax 
FUNC23_END:

; =============================================================================
; Syntax:      a*b
; Description: Multiply
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; 58 5B 0F AF C3 50 (3.0.4 Original ASM)
; 58 5B 0F AF C3 50 (FASM Re-Assemble)

db "*", 0, 0, 0, 0, 0, 0, 0
dw FUNC24_END - FUNC24_START

FUNC24_START:
pop    eax
pop    ebx
imul   eax,ebx
push   eax 
FUNC24_END:

; =============================================================================
; Syntax:      a/b
; Description: Division
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 14 bytes)
; =============================================================================
; 5B 58 0B DB 75 04 2B C0 EB 03 99 F7 FB 50 (3.0.4 Original ASM)
; 5B 58 09 DB 75 04 29 C0 EB 03 99 F7 FB 50 (FASM Re-Assemble)

db "/", 0, 0, 0, 0, 0, 0, 0
dw FUNC25_END - FUNC25_START

FUNC25_START:
pop    ebx
pop    eax
or     ebx,ebx
jne    FUNC25_START+0xa
sub    eax,eax
jmp    FUNC25_START+0xd
cdq
idiv   ebx
push   eax 
FUNC25_END:

; =============================================================================
; Syntax:      a%b
; Description: Modulo
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 14 bytes)
; =============================================================================
; 5B 58 0B DB 75 04 2B D2 EB 03 99 F7 FB 52 (3.0.4 Original ASM)
; 5B 58 09 DB 75 04 29 D2 EB 03 99 F7 FB 52 (FASM Re-Assemble)

db "%", 0, 0, 0, 0, 0, 0, 0
dw FUNC26_END - FUNC26_START

FUNC26_START:
pop    ebx
pop    eax
or     ebx,ebx
jne    FUNC26_START+0xa
sub    edx,edx
jmp    FUNC26_START+0xd
cdq
idiv   ebx
push   edx 
FUNC26_END:

; =============================================================================
; Syntax:      r
; Description: Current pixel's red channel value
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 3 bytes)
; =============================================================================
; 8B 07 50 (3.0.4 Original ASM)
; 8B 07 50 (FASM Re-Assemble)

db "r", 0, 0, 0, 0, 0, 0, 0
dw FUNC27_END - FUNC27_START

FUNC27_START:
mov    eax,DWORD [edi]
push   eax 
FUNC27_END:

; =============================================================================
; Syntax:      g
; Description: Current pixel's green channel value
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 04 50 (3.0.4 Original ASM)
; 8B 47 04 50 (FASM Re-Assemble)

db "g", 0, 0, 0, 0, 0, 0, 0
dw FUNC28_END - FUNC28_START

FUNC28_START:
mov    eax,DWORD [edi+0x4]
push   eax 
FUNC28_END:

; =============================================================================
; Syntax:      b
; Description: Current pixel's blue channel value
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 08 50 (3.0.4 Original ASM)
; 8B 47 08 50 (FASM Re-Assemble)

db "b", 0, 0, 0, 0, 0, 0, 0
dw FUNC29_END - FUNC29_START

FUNC29_START:
mov    eax,DWORD [edi+0x8]
push   eax 
FUNC29_END:

; =============================================================================
; Syntax:      a
; Description: Current pixel's alpha channel value
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 0C 50 (3.0.4 Original ASM)
; 8B 47 0C 50 (FASM Re-Assemble)

db "a", 0, 0, 0, 0, 0, 0, 0
dw FUNC30_END - FUNC30_START

FUNC30_START:
mov    eax,DWORD [edi+0xc]
push   eax 
FUNC30_END:

; =============================================================================
; Syntax:      c
; Description: Value of the current channel, whichever channel the expression is defining
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 10 bytes)
; =============================================================================
; 8B 47 58 2B 47 5C 8B 04 87 50 (3.0.4 Original ASM)
; 8B 47 58 2B 47 5C 8B 04 87 50 (FASM Re-Assemble)

db "c", 0, 0, 0, 0, 0, 0, 0
dw FUNC31_END - FUNC31_START

FUNC31_START:
mov    eax,DWORD [edi+0x58]
sub    eax,DWORD [edi+0x5c]
mov    eax,DWORD [edi+eax*4]
push   eax 
FUNC31_END:

; =============================================================================
; Syntax:      i
; Description: Y value of the YUV color space
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 55 bytes)
; =============================================================================
; 8B 77 04 8D 34 B6 03 F6 8D 34 76 8D 34 B6 8B 4F 08 8B D1 C1 E1 03 2B CA 8D 0C 8A 03 F1 8B 07 8B C8 8D 04 C0 8D 04 41 8D 04 86 99 81 E2 FF 00 00 00 03 C2 C1 F8 08 50 (3.0.4 Original ASM)
; 8B 77 04 8D 34 B6 01 F6 8D 34 76 8D 34 B6 8B 4F 08 89 CA C1 E1 03 29 D1 8D 0C 8A 01 CE 8B 07 89 C1 8D 04 C0 8D 04 41 8D 04 86 99 81 E2 FF 00 00 00 01 D0 C1 F8 08 50 (FASM Re-Assemble)

db "i", 0, 0, 0, 0, 0, 0, 0
dw FUNC32_END - FUNC32_START

FUNC32_START:
mov    esi,DWORD [edi+0x4]
lea    esi,DWORD [esi+esi*4]
add    esi,esi
lea    esi,DWORD [esi+esi*2]
lea    esi,DWORD [esi+esi*4]
mov    ecx,DWORD [edi+0x8]
mov    edx,ecx
shl    ecx,0x3
sub    ecx,edx
lea    ecx,DWORD [edx+ecx*4]
add    esi,ecx
mov    eax,DWORD [edi]
mov    ecx,eax
lea    eax,DWORD [eax+eax*8]
lea    eax,DWORD [ecx+eax*2]
lea    eax,DWORD [esi+eax*4]
cdq
and    edx,0xff
add    eax,edx
sar    eax,0x8
push   eax 
FUNC32_END:

; =============================================================================
; Syntax:      u
; Description: U value of the YUV color space
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 51 bytes)
; =============================================================================
; 8B 77 04 8B C6 8D 34 F6 8D 34 B0 F7 DE 8B 07 8B C8 8D 04 C0 8D 04 41 2B F0 8B 47 08 8B C8 C1 E0 03 2B C1 8D 04 C6 99 81 E2 FF 00 00 00 03 C2 C1 F8 08 50 (3.0.4 Original ASM)
; 8B 77 04 89 F0 8D 34 F6 8D 34 B0 F7 DE 8B 07 89 C1 8D 04 C0 8D 04 41 29 C6 8B 47 08 89 C1 C1 E0 03 29 C8 8D 04 C6 99 81 E2 FF 00 00 00 01 D0 C1 F8 08 50 (FASM Re-Assemble)

db "u", 0, 0, 0, 0, 0, 0, 0
dw FUNC33_END - FUNC33_START

FUNC33_START:
mov    esi,DWORD [edi+0x4]
mov    eax,esi
lea    esi,DWORD [esi+esi*8]
lea    esi,DWORD [eax+esi*4]
neg    esi
mov    eax,DWORD [edi]
mov    ecx,eax
lea    eax,DWORD [eax+eax*8]
lea    eax,DWORD [ecx+eax*2]
sub    esi,eax
mov    eax,DWORD [edi+0x8]
mov    ecx,eax
shl    eax,0x3
sub    eax,ecx
lea    eax,DWORD [esi+eax*8]
cdq
and    edx,0xff
add    eax,edx
sar    eax,0x8
push   eax 
FUNC33_END:

; =============================================================================
; Syntax:      v
; Description: V value of the YUV color space
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 39 bytes)
; =============================================================================
; 8B 17 8D 14 52 03 D2 8B 4F 04 8D 0C 89 2B D1 2B 57 08 8B C2 8D 14 52 8D 04 90 99 81 E2 FF 00 00 00 03 C2 C1 F8 08 50 (3.0.4 Original ASM)
; 8B 17 8D 14 52 01 D2 8B 4F 04 8D 0C 89 29 CA 2B 57 08 89 D0 8D 14 52 8D 04 90 99 81 E2 FF 00 00 00 01 D0 C1 F8 08 50 (FASM Re-Assemble)

db "v", 0, 0, 0, 0, 0, 0, 0
dw FUNC34_END - FUNC34_START

FUNC34_START:
mov    edx,DWORD [edi]
lea    edx,DWORD [edx+edx*2]
add    edx,edx
mov    ecx,DWORD [edi+0x4]
lea    ecx,DWORD [ecx+ecx*4]
sub    edx,ecx
sub    edx,DWORD [edi+0x8]
mov    eax,edx
lea    edx,DWORD [edx+edx*2]
lea    eax,DWORD [eax+edx*4]
cdq
and    edx,0xff
add    eax,edx
sar    eax,0x8
push   eax 
FUNC34_END:

; =============================================================================
; Syntax:      d
; Description: Direction (angle) of the current pixel from the center of the image, where d is an integer between 0 and 1024, inclusive
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 117 bytes)
; =============================================================================
; 8B 47 54 8B 5F 48 2B 47 50 2B 5F 44 D1 E8 D1 EB F7 D8 F7 DB 33 C9 03 47 4C 79 05 F7 D8 80 C9 04 03 5F 40 79 05 F7 DB 80 C9 03 3B C3 7E 04 80 F1 01 93 0B C0 7E 1B C1 E0 0A 99 F7 F3 74 13 66 8B 84 47 76 0D 00 00 C1 E0 09 99 BB 40 24 03 00 F7 FB 66 0F BA E1 00 73 07 F7 D8 05 00 01 00 00 66 0F BA E1 01 73 05 05 00 01 00 00 66 0F BA E1 02 73 02 F7 D8 50 (3.0.4 Original ASM)
; 8B 47 54 8B 5F 48 2B 47 50 2B 5F 44 D1 E8 D1 EB F7 D8 F7 DB 31 C9 03 47 4C 79 05 F7 D8 80 C9 04 03 5F 40 79 05 F7 DB 80 C9 03 39 D8 7E 04 80 F1 01 93 09 C0 7E 1B C1 E0 0A 99 F7 F3 74 13 66 8B 84 47 76 0D 00 00 C1 E0 09 99 BB 40 24 03 00 F7 FB 66 0F BA E1 00 73 07 F7 D8 05 00 01 00 00 66 0F BA E1 01 73 05 05 00 01 00 00 66 0F BA E1 02 73 02 F7 D8 50 (FASM Re-Assemble)

db "d", 0, 0, 0, 0, 0, 0, 0
dw FUNC35_END - FUNC35_START

FUNC35_START:
mov    eax,DWORD [edi+0x54]
mov    ebx,DWORD [edi+0x48]
sub    eax,DWORD [edi+0x50]
sub    ebx,DWORD [edi+0x44]
shr    eax,1
shr    ebx,1
neg    eax
neg    ebx
xor    ecx,ecx
add    eax,DWORD [edi+0x4c]
jns    FUNC35_START+0x20
neg    eax
or     cl,0x4
add    ebx,DWORD [edi+0x40]
jns    FUNC35_START+0x2a
neg    ebx
or     cl,0x3
cmp    eax,ebx
jle    FUNC35_START+0x32
xor    cl,0x1
xchg   ebx,eax
or     eax,eax
jle    FUNC35_START+0x51
shl    eax,0xa
cdq
div    ebx
je     FUNC35_START+0x51
mov    ax,WORD [edi+eax*2+0xd76]
shl    eax,0x9
cdq
mov    ebx,0x32440
idiv   ebx
bt     cx,0x0
jae    FUNC35_START+0x5f
neg    eax
add    eax,0x100
bt     cx,0x1
jae    FUNC35_START+0x6b
add    eax,0x100
bt     cx,0x2
jae    FUNC35_START+0x74
neg    eax
push   eax 
FUNC35_END:

; =============================================================================
; Syntax:      m
; Description: Distance (magnitude) from the center of the image to the current pixel
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 77 bytes)
; =============================================================================
; 8B 47 44 8B 5F 50 2B 47 48 2B 5F 54 D1 F8 D1 FB 03 47 40 03 5F 4C 8B D0 C1 FA 1F 33 C2 2B C2 8B D3 C1 FA 1F 33 DA 2B DA 3B C3 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 03 C3 50 (3.0.4 Original ASM)
; 8B 47 44 8B 5F 50 2B 47 48 2B 5F 54 D1 F8 D1 FB 03 47 40 03 5F 4C 89 C2 C1 FA 1F 31 D0 29 D0 89 DA C1 FA 1F 31 D3 29 D3 39 D8 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 01 D8 50 (FASM Re-Assemble)

db "m", 0, 0, 0, 0, 0, 0, 0
dw FUNC36_END - FUNC36_START

FUNC36_START:
mov    eax,DWORD [edi+0x44]
mov    ebx,DWORD [edi+0x50]
sub    eax,DWORD [edi+0x48]
sub    ebx,DWORD [edi+0x54]
sar    eax,1
sar    ebx,1
add    eax,DWORD [edi+0x40]
add    ebx,DWORD [edi+0x4c]
mov    edx,eax
sar    edx,0x1f
xor    eax,edx
sub    eax,edx
mov    edx,ebx
sar    edx,0x1f
xor    ebx,edx
sub    ebx,edx
cmp    eax,ebx
jne    FUNC36_START+0x33
mov    eax,0x6a0a
jmp    FUNC36_START+0x44
jle    FUNC36_START+0x36
xchg   ebx,eax
shl    eax,0xa
cdq
div    ebx
movzx  eax,WORD [edi+eax*2+0x1578]
mul    ebx
shrd   eax,edx,0x10
add    eax,ebx
push   eax 
FUNC36_END:

; =============================================================================
; Syntax:      R
; Description: Max channel value for r; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)

db "R", 0, 0, 0, 0, 0, 0, 0
dw FUNC37_END - FUNC37_START

FUNC37_START:
mov    eax,0xff
push   eax 
FUNC37_END:

; =============================================================================
; Syntax:      G
; Description: Max channel value for g; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)

db "G", 0, 0, 0, 0, 0, 0, 0
dw FUNC38_END - FUNC38_START

FUNC38_START:
mov    eax,0xff
push   eax 
FUNC38_END:

; =============================================================================
; Syntax:      B
; Description: Max channel value for b; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)

db "B", 0, 0, 0, 0, 0, 0, 0
dw FUNC39_END - FUNC39_START

FUNC39_START:
mov    eax,0xff
push   eax 
FUNC39_END:

; =============================================================================
; Syntax:      A
; Description: Max channel value for a; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)

db "A", 0, 0, 0, 0, 0, 0, 0
dw FUNC40_END - FUNC40_START

FUNC40_START:
mov    eax,0xff
push   eax 
FUNC40_END:

; =============================================================================
; Syntax:      I
; Description: Max channel value for i; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)

db "I", 0, 0, 0, 0, 0, 0, 0
dw FUNC41_END - FUNC41_START

FUNC41_START:
mov    eax,0xff
push   eax 
FUNC41_END:

; =============================================================================
; Syntax:      U
; Description: Max channel value for u; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)

db "U", 0, 0, 0, 0, 0, 0, 0
dw FUNC42_END - FUNC42_START

FUNC42_START:
mov    eax,0xff
push   eax 
FUNC42_END:

; =============================================================================
; Syntax:      V
; Description: Max channel value for v; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)

db "V", 0, 0, 0, 0, 0, 0, 0
dw FUNC43_END - FUNC43_START

FUNC43_START:
mov    eax,0xff
push   eax 
FUNC43_END:

; =============================================================================
; Syntax:      C
; Description: Max channel value for c; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)

db "C", 0, 0, 0, 0, 0, 0, 0
dw FUNC44_END - FUNC44_START

FUNC44_START:
mov    eax,0xff
push   eax 
FUNC44_END:

; =============================================================================
; Syntax:      D
; Description: Range of angles within the image, where dmin is always 0 and D is always 1024
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; B8 00 04 00 00 50 (3.0.4 Original ASM)
; B8 00 04 00 00 50 (FASM Re-Assemble)

db "D", 0, 0, 0, 0, 0, 0, 0
dw FUNC45_END - FUNC45_START

FUNC45_START:
mov    eax,0x400
push   eax 
FUNC45_END:

; =============================================================================
; Syntax:      M
; Description: Range of magnitudes with the image, where mmin is always 0 and M is always one half the diagonal size of the image
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 53 bytes)
; =============================================================================
; 8B 47 48 8B 5F 54 2B 47 44 2B 5F 50 D1 F8 D1 FB 3B C3 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 03 C3 50 (3.0.4 Original ASM)
; 8B 47 48 8B 5F 54 2B 47 44 2B 5F 50 D1 F8 D1 FB 39 D8 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 01 D8 50 (FASM Re-Assemble)

db "M", 0, 0, 0, 0, 0, 0, 0
dw FUNC46_END - FUNC46_START

FUNC46_START:
mov    eax,DWORD [edi+0x48]
mov    ebx,DWORD [edi+0x54]
sub    eax,DWORD [edi+0x44]
sub    ebx,DWORD [edi+0x50]
sar    eax,1
sar    ebx,1
cmp    eax,ebx
jne    FUNC46_START+0x1b
mov    eax,0x6a0a
jmp    FUNC46_START+0x2c
jle    FUNC46_START+0x1e
xchg   ebx,eax
shl    eax,0xa
cdq
div    ebx
movzx  eax,WORD [edi+eax*2+0x1578]
mul    ebx
shrd   eax,edx,0x10
add    eax,ebx
push   eax 
FUNC46_END:

; =============================================================================
; Syntax:      x
; Description: x coordinate of the current pixel
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 40 50 (3.0.4 Original ASM)
; 8B 47 40 50 (FASM Re-Assemble)

db "x", 0, 0, 0, 0, 0, 0, 0
dw FUNC47_END - FUNC47_START

FUNC47_START:
mov    eax,DWORD [edi+0x40]
push   eax 
FUNC47_END:

; =============================================================================
; Syntax:      y
; Description: y coordinate of the current pixel
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 4C 50 (3.0.4 Original ASM)
; 8B 47 4C 50 (FASM Re-Assemble)

db "y", 0, 0, 0, 0, 0, 0, 0
dw FUNC48_END - FUNC48_START

FUNC48_START:
mov    eax,DWORD [edi+0x4c]
push   eax 
FUNC48_END:

; =============================================================================
; Syntax:      z
; Description: Channel index for the current expression
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 58 50 (3.0.4 Original ASM)
; 8B 47 58 50 (FASM Re-Assemble)

db "z", 0, 0, 0, 0, 0, 0, 0
dw FUNC49_END - FUNC49_START

FUNC49_START:
mov    eax,DWORD [edi+0x58]
push   eax 
FUNC49_END:

; =============================================================================
; Syntax:      X
; Description: Range of horizontal coordinates over the width of the image
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 48 50 (3.0.4 Original ASM)
; 8B 47 48 50 (FASM Re-Assemble)

db "X", 0, 0, 0, 0, 0, 0, 0
dw FUNC50_END - FUNC50_START

FUNC50_START:
mov    eax,DWORD [edi+0x48]
push   eax 
FUNC50_END:

; =============================================================================
; Syntax:      Y
; Description: Range of vertical coordinates over the height of the image
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 54 50 (3.0.4 Original ASM)
; 8B 47 54 50 (FASM Re-Assemble)

db "Y", 0, 0, 0, 0, 0, 0, 0
dw FUNC51_END - FUNC51_START

FUNC51_START:
mov    eax,DWORD [edi+0x54]
push   eax 
FUNC51_END:

; =============================================================================
; Syntax:      Z
; Description: Range of channel indexes within one pixel
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 60 50 (3.0.4 Original ASM)
; 8B 47 60 50 (FASM Re-Assemble)

db "Z", 0, 0, 0, 0, 0, 0, 0
dw FUNC52_END - FUNC52_START

FUNC52_START:
mov    eax,DWORD [edi+0x60]
push   eax 
FUNC52_END:

; =============================================================================
; Syntax:      xmin
; Description: Smallest x coodinate; always 0
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 44 50 (3.0.4 Original ASM)
; 8B 47 44 50 (FASM Re-Assemble)

db "xmin", 0, 0, 0, 0
dw FUNC53_END - FUNC53_START

FUNC53_START:
mov    eax,DWORD [edi+0x44]
push   eax 
FUNC53_END:

; =============================================================================
; Syntax:      ymin
; Description: Smallest y coodinate; always 0
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 50 50 (3.0.4 Original ASM)
; 8B 47 50 50 (FASM Re-Assemble)

db "ymin", 0, 0, 0, 0
dw FUNC54_END - FUNC54_START

FUNC54_START:
mov    eax,DWORD [edi+0x50]
push   eax 
FUNC54_END:

; =============================================================================
; Syntax:      zmin
; Description: Smallest z coodinate; always 0
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 5C 50 (3.0.4 Original ASM)
; 8B 47 5C 50 (FASM Re-Assemble)

db "zmin", 0, 0, 0, 0
dw FUNC55_END - FUNC55_START

FUNC55_START:
mov    eax,DWORD [edi+0x5c]
push   eax 
FUNC55_END:

; =============================================================================
; Syntax:      rmin
; Description: Smallest red value; always 0
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 3 bytes)
; =============================================================================
; 2B C0 50 (3.0.4 Original ASM)
; 29 C0 50 (FASM Re-Assemble)

db "rmin", 0, 0, 0, 0
dw FUNC56_END - FUNC56_START

FUNC56_START:
sub    eax,eax
push   eax 
FUNC56_END:

; =============================================================================
; Syntax:      tmax
; Description: Total number of frames (used only in Adobe Premiere). always returns 1. 
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; B8 01 00 00 00 50 (3.0.4 Original ASM)
; B8 01 00 00 00 50 (FASM Re-Assemble)

db "tmax", 0, 0, 0, 0
dw FUNC57_END - FUNC57_START

FUNC57_START:
mov    eax,0x1
push   eax 
FUNC57_END:

; =============================================================================
; Syntax:      min(a,b)
; Description: Lesser of a and b
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 9 bytes)
; =============================================================================
; 58 59 3B C1 7C 02 8B C1 50 (3.0.4 Original ASM)
; 58 59 39 C8 7C 02 89 C8 50 (FASM Re-Assemble)

db "min", 0, 0, 0, 0, 0
dw FUNC58_END - FUNC58_START

FUNC58_START:
pop    eax
pop    ecx
cmp    eax,ecx
jl     FUNC58_START+0x8
mov    eax,ecx
push   eax 
FUNC58_END:

; =============================================================================
; Syntax:      max(a,b)
; Description: Greater of a and b
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 9 bytes)
; =============================================================================
; 58 59 3B C1 7F 02 8B C1 50 (3.0.4 Original ASM)
; 58 59 39 C8 7F 02 89 C8 50 (FASM Re-Assemble)

db "max", 0, 0, 0, 0, 0
dw FUNC59_END - FUNC59_START

FUNC59_START:
pop    eax
pop    ecx
cmp    eax,ecx
jg     FUNC59_START+0x8
mov    eax,ecx
push   eax 
FUNC59_END:

; =============================================================================
; Syntax:      abs(a)
; Description: Absolute value of a.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 7 bytes)
; =============================================================================
; 58 99 33 C2 2B C2 50 (3.0.4 Original ASM)
; 58 99 31 D0 29 D0 50 (FASM Re-Assemble)

db "abs", 0, 0, 0, 0, 0
dw FUNC60_END - FUNC60_START

FUNC60_START:
pop    eax
cdq
xor    eax,edx
sub    eax,edx
push   eax 
FUNC60_END:

; =============================================================================
; Syntax:      add(a,b,c)
; Description: Sum of a and b, or c, whichever is lesser
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 12 bytes)
; =============================================================================
; 59 5B 58 03 C3 3B C1 7C 02 8B C1 50 (3.0.4 Original ASM)
; 59 5B 58 01 D8 39 C8 7C 02 89 C8 50 (FASM Re-Assemble)

db "add", 0, 0, 0, 0, 0
dw FUNC61_END - FUNC61_START

FUNC61_START:
pop    ecx
pop    ebx
pop    eax
add    eax,ebx
cmp    eax,ecx
jl     FUNC61_START+0xb
mov    eax,ecx
push   eax 
FUNC61_END:

; =============================================================================
; Syntax:      sub(a,b,c)
; Description: Difference of a and b, or c, whichever is greater
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 12 bytes)
; =============================================================================
; 59 5B 58 2B C3 3B C1 7F 02 8B C1 50 (3.0.4 Original ASM)
; 59 5B 58 29 D8 39 C8 7F 02 89 C8 50 (FASM Re-Assemble)

db "sub", 0, 0, 0, 0, 0
dw FUNC62_END - FUNC62_START

FUNC62_START:
pop    ecx
pop    ebx
pop    eax
sub    eax,ebx
cmp    eax,ecx
jg     FUNC62_START+0xb
mov    eax,ecx
push   eax 
FUNC62_END:

; =============================================================================
; Syntax:      dif(a,b)
; Description: Absolute value of the difference of a and b
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 10 bytes)
; =============================================================================
; 58 5B 2B C3 99 33 C2 2B C2 50 (3.0.4 Original ASM)
; 58 5B 29 D8 99 31 D0 29 D0 50 (FASM Re-Assemble)

db "dif", 0, 0, 0, 0, 0
dw FUNC63_END - FUNC63_START

FUNC63_START:
pop    eax
pop    ebx
sub    eax,ebx
cdq
xor    eax,edx
sub    eax,edx
push   eax 
FUNC63_END:

; =============================================================================
; Syntax:      mix(a,b,n,d)
; Description: Mixture of a and b by fraction n/d, a*n/d+b*(d-n)/d
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 32 bytes)
; =============================================================================
; 5B 59 5A 5E 0B DB B8 00 00 00 00 74 12 8B C3 2B C1 F7 EA F7 FB 8B E8 8B C1 F7 EE F7 FB 03 C5 50 (3.0.4 Original ASM)
; 5B 59 5A 5E 09 DB B8 00 00 00 00 74 12 89 D8 29 C8 F7 EA F7 FB 89 C5 89 C8 F7 EE F7 FB 01 E8 50 (FASM Re-Assemble)

db "mix", 0, 0, 0, 0, 0
dw FUNC64_END - FUNC64_START

FUNC64_START:
pop    ebx
pop    ecx
pop    edx
pop    esi
or     ebx,ebx
mov    eax,0x0
je     FUNC64_START+0x1f
mov    eax,ebx
sub    eax,ecx
imul   edx
idiv   ebx
mov    ebp,eax
mov    eax,ecx
imul   esi
idiv   ebx
add    eax,ebp
push   eax 
FUNC64_END:

; =============================================================================
; Syntax:      scl(a,il,ih,ol,oh)
; Description: Scale a from input range (il to ih) to output range (ol to oh)
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 39 bytes)
; =============================================================================
; 8B 6C 24 08 8B 4C 24 0C 2B E9 8B C5 74 15 8B 04 24 8B 74 24 04 2B C6 8B 54 24 10 2B D1 F7 EA F7 FD 03 C6 83 C4 14 50 (3.0.4 Original ASM)
; 8B 6C 24 08 8B 4C 24 0C 29 CD 89 E8 74 15 8B 04 24 8B 74 24 04 29 F0 8B 54 24 10 29 CA F7 EA F7 FD 01 F0 83 C4 14 50 (FASM Re-Assemble)

db "scl", 0, 0, 0, 0, 0
dw FUNC65_END - FUNC65_START

FUNC65_START:
mov    ebp,DWORD [esp+0x8]
mov    ecx,DWORD [esp+0xc]
sub    ebp,ecx
mov    eax,ebp
je     FUNC65_START+0x23
mov    eax,DWORD [esp]
mov    esi,DWORD [esp+0x4]
sub    eax,esi
mov    edx,DWORD [esp+0x10]
sub    edx,ecx
imul   edx
idiv   ebp
add    eax,esi
add    esp,0x14
push   eax 
FUNC65_END:

; =============================================================================
; Syntax:      sqr(x)
; Description: Square root of x
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 31 bytes)
; =============================================================================
; 5B 83 FB 01 7E 18 8B CB D1 EB B8 02 00 00 00 3B D8 7E 0B 8B C1 99 F7 F3 03 D8 D1 EB EB F1 53 (3.0.4 Original ASM)
; 5B 83 FB 01 7E 18 89 D9 D1 EB B8 02 00 00 00 39 C3 7E 0B 89 C8 99 F7 F3 01 C3 D1 EB EB F1 53 (FASM Re-Assemble)

db "sqr", 0, 0, 0, 0, 0
dw FUNC66_END - FUNC66_START

FUNC66_START:
pop    ebx
cmp    ebx,0x1
jle    FUNC66_START+0x1e
mov    ecx,ebx
shr    ebx,1
mov    eax,0x2
cmp    ebx,eax
jle    FUNC66_START+0x1e
mov    eax,ecx
cdq
div    ebx
add    ebx,eax
shr    ebx,1
jmp    FUNC66_START+0xf
push   ebx 
FUNC66_END:

; =============================================================================
; Syntax:      rnd(a,b)
; Description: Random number between a and b, inclusive
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 82 bytes)
; =============================================================================
; 8D 9F 94 00 00 00 66 8B 43 02 66 40 66 3D 38 00 75 04 66 B8 01 00 66 89 43 02 66 8B 03 66 40 66 3D 38 00 75 04 66 B8 01 00 66 89 03 0F B7 03 8D 4C 83 04 8B 01 0F B6 53 02 2B 44 93 04 89 01 5E 59 2B F1 79 04 33 C0 EB 08 46 2B D2 F7 F6 8D 04 11 50 (3.0.4 Original ASM)
; 8D 9F 94 00 00 00 66 8B 43 02 66 40 66 83 F8 38 75 04 66 B8 01 00 66 89 43 02 66 8B 03 66 40 66 83 F8 38 75 04 66 B8 01 00 66 89 03 0F B7 03 8D 4C 83 04 8B 01 0F B6 53 02 2B 44 93 04 89 01 5E 59 29 CE 79 04 31 C0 EB 08 46 29 D2 F7 F6 8D 04 11 50 (FASM Re-Assemble)

db "rnd", 0, 0, 0, 0, 0
dw FUNC67_END - FUNC67_START

FUNC67_START:
lea    ebx,DWORD [edi+0x94]
mov    ax,WORD [ebx+0x2]
inc    ax
cmp    ax,0x38
jne    FUNC67_START+0x16
mov    ax,0x1
mov    WORD [ebx+0x2],ax
mov    ax,WORD [ebx]
inc    ax
cmp    ax,0x38
jne    FUNC67_START+0x29
mov    ax,0x1
mov    WORD [ebx],ax
movzx  eax,WORD [ebx]
lea    ecx,DWORD [ebx+eax*4+0x4]
mov    eax,DWORD [ecx]
movzx  edx,BYTE [ebx+0x2]
sub    eax,DWORD [ebx+edx*4+0x4]
mov    DWORD [ecx],eax
pop    esi
pop    ecx
sub    esi,ecx
jns    FUNC67_START+0x49
xor    eax,eax
jmp    FUNC67_START+0x51
inc    esi
sub    edx,edx
div    esi
lea    eax,DWORD [ecx+edx*1]
push   eax 
FUNC67_END:

; =============================================================================
; Syntax:      rst(i)
; Description: UNDOCUMENTED! Sets a seed for rnd(). Parameter i has the range of 0..32767
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 10 bytes)
; =============================================================================
; 58 89 87 78 45 00 00 2B C0 50 (3.0.4 Original ASM)
; 58 89 87 78 45 00 00 29 C0 50 (FASM Re-Assemble)

db "rst", 0, 0, 0, 0, 0
dw FUNC68_END - FUNC68_START

FUNC68_START:
pop    eax
mov    DWORD [edi+0x4578],eax
sub    eax,eax
push   eax 
FUNC68_END:

; =============================================================================
; Syntax:      ctl(i)
; Description: Value of slider i, where i is an integer between 0 and 7, inclusive.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 17 bytes)
; =============================================================================
; 5B 2B C0 0B DB 7C 09 83 FB 07 7F 04 8B 44 9F 64 50 (3.0.4 Original ASM)
; 5B 29 C0 09 DB 7C 09 83 FB 07 7F 04 8B 44 9F 64 50 (FASM Re-Assemble)

db "ctl", 0, 0, 0, 0, 0
dw FUNC69_END - FUNC69_START

FUNC69_START:
pop    ebx
sub    eax,eax
or     ebx,ebx
jl     FUNC69_START+0x10
cmp    ebx,0x7
jg     FUNC69_START+0x10
mov    eax,DWORD [edi+ebx*4+0x64]
push   eax 
FUNC69_END:

; =============================================================================
; Syntax:      val(i,a,b)
; Description: Value of slider i, mapped onto the range a to b
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 34 bytes)
; =============================================================================
; 59 5E 5B 2B C0 0B DB 7C 18 83 FB 07 7F 13 8B 44 9F 64 2B CE 0F AF C1 B9 FF 00 00 00 99 F7 F9 03 C6 50 (3.0.4 Original ASM)
; 59 5E 5B 29 C0 09 DB 7C 18 83 FB 07 7F 13 8B 44 9F 64 29 F1 0F AF C1 B9 FF 00 00 00 99 F7 F9 01 F0 50 (FASM Re-Assemble)

db "val", 0, 0, 0, 0, 0
dw FUNC70_END - FUNC70_START

FUNC70_START:
pop    ecx
pop    esi
pop    ebx
sub    eax,eax
or     ebx,ebx
jl     FUNC70_START+0x21
cmp    ebx,0x7
jg     FUNC70_START+0x21
mov    eax,DWORD [edi+ebx*4+0x64]
sub    ecx,esi
imul   eax,ecx
mov    ecx,0xff
cdq
idiv   ecx
add    eax,esi
push   eax 
FUNC70_END:

; =============================================================================
; Syntax:      map(i,n)
; Description: Item n from mapping table i, where i is an integer between 0 and 3, inclusive, and n is and integer between 0 and 255, inclusive
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 48 bytes)
; =============================================================================
; 58 5B 0B DB 7C 27 83 FB 03 7F 22 0B C0 7D 04 2B C0 EB 0B B9 FF 00 00 00 3B C1 7E 02 8B C1 C1 E3 08 03 DF 0F B6 84 18 78 01 00 00 EB 02 2B C0 50 (3.0.4 Original ASM)
; 58 5B 09 DB 7C 27 83 FB 03 7F 22 09 C0 7D 04 29 C0 EB 0B B9 FF 00 00 00 39 C8 7E 02 89 C8 C1 E3 08 01 FB 0F B6 84 18 78 01 00 00 EB 02 29 C0 50 (FASM Re-Assemble)

db "map", 0, 0, 0, 0, 0
dw FUNC71_END - FUNC71_START

FUNC71_START:
pop    eax
pop    ebx
or     ebx,ebx
jl     FUNC71_START+0x2d
cmp    ebx,0x3
jg     FUNC71_START+0x2d
or     eax,eax
jge    FUNC71_START+0x13
sub    eax,eax
jmp    FUNC71_START+0x1e
mov    ecx,0xff
cmp    eax,ecx
jle    FUNC71_START+0x1e
mov    eax,ecx
shl    ebx,0x8
add    ebx,edi
movzx  eax,BYTE [eax+ebx*1+0x178]
jmp    FUNC71_START+0x2f
sub    eax,eax
push   eax 
FUNC71_END:

; =============================================================================
; Syntax:      src(x,y,z)
; Description: Channel z for the pixel at coordinates x,y
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 94 bytes)
; =============================================================================
; 5A 59 5B 3B 57 5C 7C 53 3B 57 60 7D 4E 8B 77 44 3B DE 7D 02 8B DE 8B 47 48 3B D8 7C 03 8D 58 FF 8B 47 50 3B C8 7D 02 8B C8 8B 47 54 3B C8 7C 03 8D 48 FF 8B 87 A8 45 00 00 2B 57 5C 03 C2 2B 4F 50 0F AF 8F A0 45 00 00 03 C1 2B DE 8B 57 60 2B 57 5C 0F AF DA 0F B6 04 18 EB 02 2B C0 50 (3.0.4 Original ASM)
; 5A 59 5B 3B 57 5C 7C 53 3B 57 60 7D 4E 8B 77 44 39 F3 7D 02 89 F3 8B 47 48 39 C3 7C 03 8D 58 FF 8B 47 50 39 C1 7D 02 89 C1 8B 47 54 39 C1 7C 03 8D 48 FF 8B 87 A8 45 00 00 2B 57 5C 01 D0 2B 4F 50 0F AF 8F A0 45 00 00 01 C8 29 F3 8B 57 60 2B 57 5C 0F AF DA 0F B6 04 18 EB 02 29 C0 50 (FASM Re-Assemble)

db "src", 0, 0, 0, 0, 0
dw FUNC72_END - FUNC72_START

FUNC72_START:
pop    edx
pop    ecx
pop    ebx
cmp    edx,DWORD [edi+0x5c]
jl     FUNC72_START+0x5b
cmp    edx,DWORD [edi+0x60]
jge    FUNC72_START+0x5b
mov    esi,DWORD [edi+0x44]
cmp    ebx,esi
jge    FUNC72_START+0x16
mov    ebx,esi
mov    eax,DWORD [edi+0x48]
cmp    ebx,eax
jl     FUNC72_START+0x20
lea    ebx,DWORD [eax-0x1]
mov    eax,DWORD [edi+0x50]
cmp    ecx,eax
jge    FUNC72_START+0x29
mov    ecx,eax
mov    eax,DWORD [edi+0x54]
cmp    ecx,eax
jl     FUNC72_START+0x33
lea    ecx,DWORD [eax-0x1]
mov    eax,DWORD [edi+0x45a8]
sub    edx,DWORD [edi+0x5c]
add    eax,edx
sub    ecx,DWORD [edi+0x50]
imul   ecx,DWORD [edi+0x45a0]
add    eax,ecx
sub    ebx,esi
mov    edx,DWORD [edi+0x60]
sub    edx,DWORD [edi+0x5c]
imul   ebx,edx
movzx  eax,BYTE [eax+ebx*1]
jmp    FUNC72_START+0x5d
sub    eax,eax
push   eax 
FUNC72_END:

; =============================================================================
; Syntax:      cnv(m11,m12,m13,m21,m22,m23,m31,m32,m33,d)
; Description: "Convolve". Applies a convolution matrix and divides with d.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 187 bytes)
; =============================================================================
; 8B 04 24 0B C0 0F 84 AC 00 00 00 8B 8F A8 45 00 00 8B 47 58 2B 47 5C 03 C8 8B 47 4C 48 2B 47 50 F7 A7 A0 45 00 00 03 C8 8B 47 40 48 2B 47 44 8B 5F 60 2B 5F 5C F7 E3 03 C8 2B C0 8B D1 0F B6 32 0F AF 74 24 24 03 C6 03 D3 0F B6 32 0F AF 74 24 20 03 C6 03 D3 0F B6 32 0F AF 74 24 1C 03 C6 03 8F A0 45 00 00 8B D1 0F B6 32 0F AF 74 24 18 03 C6 03 D3 0F B6 32 0F AF 74 24 14 03 C6 03 D3 0F B6 32 0F AF 74 24 10 03 C6 03 8F A0 45 00 00 8B D1 0F B6 32 0F AF 74 24 0C 03 C6 03 D3 0F B6 32 0F AF 74 24 08 03 C6 03 D3 0F B6 32 0F AF 74 24 04 03 C6 99 F7 3C 24 83 C4 28 50 (3.0.4 Original ASM)
; 8B 04 24 09 C0 0F 84 AC 00 00 00 8B 8F A8 45 00 00 8B 47 58 2B 47 5C 01 C1 8B 47 4C 48 2B 47 50 F7 A7 A0 45 00 00 01 C1 8B 47 40 48 2B 47 44 8B 5F 60 2B 5F 5C F7 E3 01 C1 29 C0 89 CA 0F B6 32 0F AF 74 24 24 01 F0 01 DA 0F B6 32 0F AF 74 24 20 01 F0 01 DA 0F B6 32 0F AF 74 24 1C 01 F0 03 8F A0 45 00 00 89 CA 0F B6 32 0F AF 74 24 18 01 F0 01 DA 0F B6 32 0F AF 74 24 14 01 F0 01 DA 0F B6 32 0F AF 74 24 10 01 F0 03 8F A0 45 00 00 89 CA 0F B6 32 0F AF 74 24 0C 01 F0 01 DA 0F B6 32 0F AF 74 24 08 01 F0 01 DA 0F B6 32 0F AF 74 24 04 01 F0 99 F7 3C 24 83 C4 28 50 (FASM Re-Assemble)

db "cnv", 0, 0, 0, 0, 0
dw FUNC73_END - FUNC73_START

FUNC73_START:
mov    eax,DWORD [esp]
or     eax,eax
je     FUNC73_START+0xb7
mov    ecx,DWORD [edi+0x45a8]
mov    eax,DWORD [edi+0x58]
sub    eax,DWORD [edi+0x5c]
add    ecx,eax
mov    eax,DWORD [edi+0x4c]
dec    eax
sub    eax,DWORD [edi+0x50]
mul    DWORD [edi+0x45a0]
add    ecx,eax
mov    eax,DWORD [edi+0x40]
dec    eax
sub    eax,DWORD [edi+0x44]
mov    ebx,DWORD [edi+0x60]
sub    ebx,DWORD [edi+0x5c]
mul    ebx
add    ecx,eax
sub    eax,eax
mov    edx,ecx
movzx  esi,BYTE  [edx]
imul   esi,DWORD [esp+0x24]
add    eax,esi
add    edx,ebx
movzx  esi,BYTE  [edx]
imul   esi,DWORD [esp+0x20]
add    eax,esi
add    edx,ebx
movzx  esi,BYTE  [edx]
imul   esi,DWORD [esp+0x1c]
add    eax,esi
add    ecx,DWORD [edi+0x45a0]
mov    edx,ecx
movzx  esi,BYTE  [edx]
imul   esi,DWORD [esp+0x18]
add    eax,esi
add    edx,ebx
movzx  esi,BYTE  [edx]
imul   esi,DWORD [esp+0x14]
add    eax,esi
add    edx,ebx
movzx  esi,BYTE  [edx]
imul   esi,DWORD [esp+0x10]
add    eax,esi
add    ecx,DWORD [edi+0x45a0]
mov    edx,ecx
movzx  esi,BYTE  [edx]
imul   esi,DWORD [esp+0xc]
add    eax,esi
add    edx,ebx
movzx  esi,BYTE  [edx]
imul   esi,DWORD [esp+0x8]
add    eax,esi
add    edx,ebx
movzx  esi,BYTE  [edx]
imul   esi,DWORD [esp+0x4]
add    eax,esi
cdq
idiv   DWORD [esp]
add    esp,0x28
push   eax 
FUNC73_END:

; =============================================================================
; Syntax:      rad(d,m,z)
; Description: Channel z in the source image, which is m units away, at an angle of d, from the center of the image
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 158 bytes)
; =============================================================================
; 5E 8B 04 24 8B 5C 24 04 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 8B 4F 48 0F AC D0 0E 2B 4F 44 D1 E9 03 C8 0B C9 7D 04 2B C9 EB 0A 8B 47 48 3B C8 7C 03 8D 48 FF 58 5B 81 EB 00 01 00 00 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 8B 5F 54 0F AC D0 0E 2B 5F 50 D1 EB 03 D8 0B DB 7D 04 2B DB EB 0A 8B 47 54 3B D8 7C 03 8D 58 FF 8B 57 60 2B 57 5C 03 B7 A8 45 00 00 0F AF 9F A0 45 00 00 0F AF CA 33 C0 03 D9 8A 04 1E 50 (3.0.4 Original ASM)
; 5E 8B 04 24 8B 5C 24 04 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 8B 4F 48 0F AC D0 0E 2B 4F 44 D1 E9 01 C1 09 C9 7D 04 29 C9 EB 0A 8B 47 48 39 C1 7C 03 8D 48 FF 58 5B 81 EB 00 01 00 00 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 8B 5F 54 0F AC D0 0E 2B 5F 50 D1 EB 01 C3 09 DB 7D 04 29 DB EB 0A 8B 47 54 39 C3 7C 03 8D 58 FF 8B 57 60 2B 57 5C 03 B7 A8 45 00 00 0F AF 9F A0 45 00 00 0F AF CA 31 C0 01 CB 8A 04 1E 50 (FASM Re-Assemble)

db "rad", 0, 0, 0, 0, 0
dw FUNC74_END - FUNC74_START

FUNC74_START:
pop    esi
mov    eax,DWORD [esp]
mov    ebx,DWORD [esp+0x4]
and    ebx,0x3ff
movsx  ebx,WORD [edi+ebx*2+0x578]
imul   ebx
add    eax,0x1fff
adc    edx,0x0
mov    ecx,DWORD [edi+0x48]
shrd   eax,edx,0xe
sub    ecx,DWORD [edi+0x44]
shr    ecx,1
add    ecx,eax
or     ecx,ecx
jge    FUNC74_START+0x36
sub    ecx,ecx
jmp    FUNC74_START+0x40
mov    eax,DWORD [edi+0x48]
cmp    ecx,eax
jl     FUNC74_START+0x40
lea    ecx,DWORD [eax-0x1]
pop    eax
pop    ebx
sub    ebx,0x100
and    ebx,0x3ff
movsx  ebx,WORD [edi+ebx*2+0x578]
imul   ebx
add    eax,0x1fff
adc    edx,0x0
mov    ebx,DWORD [edi+0x54]
shrd   eax,edx,0xe
sub    ebx,DWORD [edi+0x50]
shr    ebx,1
add    ebx,eax
or     ebx,ebx
jge    FUNC74_START+0x76
sub    ebx,ebx
jmp    FUNC74_START+0x80
mov    eax,DWORD [edi+0x54]
cmp    ebx,eax
jl     FUNC74_START+0x80
lea    ebx,DWORD [eax-0x1]
mov    edx,DWORD [edi+0x60]
sub    edx,DWORD [edi+0x5c]
add    esi,DWORD [edi+0x45a8]
imul   ebx,DWORD [edi+0x45a0]
imul   ecx,edx
xor    eax,eax
add    ebx,ecx
mov    al,BYTE  [esi+ebx*1]
push   eax 
FUNC74_END:

; =============================================================================
; Syntax:      c2d(x,y)
; Description: Angle displacement of the pixel at coordinates x,y
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 97 bytes)
; =============================================================================
; 58 5B 33 C9 0B C0 7D 05 F7 D8 80 C9 04 0B DB 7D 05 F7 DB 80 C9 03 3B C3 7E 04 80 F1 01 93 0B C0 7E 1B C1 E0 0A 99 F7 F3 74 13 66 8B 84 47 76 0D 00 00 C1 E0 09 99 BB 40 24 03 00 F7 FB 66 0F BA E1 00 73 07 F7 D8 05 00 01 00 00 66 0F BA E1 01 73 05 05 00 01 00 00 66 0F BA E1 02 73 02 F7 D8 50 (3.0.4 Original ASM)
; 58 5B 31 C9 09 C0 7D 05 F7 D8 80 C9 04 09 DB 7D 05 F7 DB 80 C9 03 39 D8 7E 04 80 F1 01 93 09 C0 7E 1B C1 E0 0A 99 F7 F3 74 13 66 8B 84 47 76 0D 00 00 C1 E0 09 99 BB 40 24 03 00 F7 FB 66 0F BA E1 00 73 07 F7 D8 05 00 01 00 00 66 0F BA E1 01 73 05 05 00 01 00 00 66 0F BA E1 02 73 02 F7 D8 50 (FASM Re-Assemble)

db "c2d", 0, 0, 0, 0, 0
dw FUNC75_END - FUNC75_START

FUNC75_START:
pop    eax
pop    ebx
xor    ecx,ecx
or     eax,eax
jge    FUNC75_START+0xd
neg    eax
or     cl,0x4
or     ebx,ebx
jge    FUNC75_START+0x16
neg    ebx
or     cl,0x3
cmp    eax,ebx
jle    FUNC75_START+0x1e
xor    cl,0x1
xchg   ebx,eax
or     eax,eax
jle    FUNC75_START+0x3d
shl    eax,0xa
cdq
div    ebx
je     FUNC75_START+0x3d
mov    ax,WORD [edi+eax*2+0xd76]
shl    eax,0x9
cdq
mov    ebx,0x32440
idiv   ebx
bt     cx,0x0
jae    FUNC75_START+0x4b
neg    eax
add    eax,0x100
bt     cx,0x1
jae    FUNC75_START+0x57
add    eax,0x100
bt     cx,0x2
jae    FUNC75_START+0x60
neg    eax
push   eax 
FUNC75_END:

; =============================================================================
; Syntax:      c2m(x,y)
; Description: Magnitude displacement of the pixel at coordinates x,y
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 57 bytes)
; =============================================================================
; 5B 58 8B D0 C1 FA 1F 33 C2 2B C2 8B D3 C1 FA 1F 33 DA 2B DA 3B C3 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 03 C3 50 (3.0.4 Original ASM)
; 5B 58 89 C2 C1 FA 1F 31 D0 29 D0 89 DA C1 FA 1F 31 D3 29 D3 39 D8 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 01 D8 50 (FASM Re-Assemble)

db "c2m", 0, 0, 0, 0, 0
dw FUNC76_END - FUNC76_START

FUNC76_START:
pop    ebx
pop    eax
mov    edx,eax
sar    edx,0x1f
xor    eax,edx
sub    eax,edx
mov    edx,ebx
sar    edx,0x1f
xor    ebx,edx
sub    ebx,edx
cmp    eax,ebx
jne    FUNC76_START+0x1f
mov    eax,0x6a0a
jmp    FUNC76_START+0x30
jle    FUNC76_START+0x22
xchg   ebx,eax
shl    eax,0xa
cdq
div    ebx
movzx  eax,WORD [edi+eax*2+0x1578]
mul    ebx
shrd   eax,edx,0x10
add    eax,ebx
push   eax 
FUNC76_END:

; =============================================================================
; Syntax:      r2x(d,m)
; Description: x displacement of the pixel m units away, at an angle of d, from an arbitrary center
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 31 bytes)
; =============================================================================
; 58 5B 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 0F AC D0 0E 50 (3.0.4 Original ASM)
; 58 5B 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 0F AC D0 0E 50 (FASM Re-Assemble)

db "r2x", 0, 0, 0, 0, 0
dw FUNC77_END - FUNC77_START

FUNC77_START:
pop    eax
pop    ebx
and    ebx,0x3ff
movsx  ebx,WORD [edi+ebx*2+0x578]
imul   ebx
add    eax,0x1fff
adc    edx,0x0
shrd   eax,edx,0xe
push   eax 
FUNC77_END:

; =============================================================================
; Syntax:      r2y(d,m)
; Description: y displacement of the pixel m units away, at an angle of d, from an arbitrary center
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 37 bytes)
; =============================================================================
; 58 5B 81 EB 00 01 00 00 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 0F AC D0 0E 50 (3.0.4 Original ASM)
; 58 5B 81 EB 00 01 00 00 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 0F AC D0 0E 50 (FASM Re-Assemble)

db "r2y", 0, 0, 0, 0, 0
dw FUNC78_END - FUNC78_START

FUNC78_START:
pop    eax
pop    ebx
sub    ebx,0x100
and    ebx,0x3ff
movsx  ebx,WORD [edi+ebx*2+0x578]
imul   ebx
add    eax,0x1fff
adc    edx,0x0
shrd   eax,edx,0xe
push   eax 
FUNC78_END:

; =============================================================================
; Syntax:      sin(x)
; Description: Sine function of x, where x is an integer between 0 and 1024, inclusive, and the value returned is an integer between -512 and 512, inclusive (Windows) or -1024 and 1024, inclusive (Mac OS)
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 32 bytes)
; =============================================================================
; 58 2D 00 01 00 00 8B D8 C1 FB 1F 33 C3 2B C3 25 FF 03 00 00 0F BF 84 47 78 05 00 00 C1 F8 05 50 (3.0.4 Original ASM)
; 58 2D 00 01 00 00 89 C3 C1 FB 1F 31 D8 29 D8 25 FF 03 00 00 0F BF 84 47 78 05 00 00 C1 F8 05 50 (FASM Re-Assemble)

db "sin", 0, 0, 0, 0, 0
dw FUNC79_END - FUNC79_START

FUNC79_START:
pop    eax
sub    eax,0x100
mov    ebx,eax
sar    ebx,0x1f
xor    eax,ebx
sub    eax,ebx
and    eax,0x3ff
movsx  eax,WORD [edi+eax*2+0x578]
sar    eax,0x5
push   eax 
FUNC79_END:

; =============================================================================
; Syntax:      cos(x)
; Description: Cosine function of x, where x is an integer between 0 and 1024, inclusive, and the value returned is an integer between -512 and 512, inclusive (Windows) or -1024 and 1024, inclusive (Mac OS)
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 27 bytes)
; =============================================================================
; 58 8B D8 C1 FB 1F 33 C3 2B C3 25 FF 03 00 00 0F BF 84 47 78 05 00 00 C1 F8 05 50 (3.0.4 Original ASM)
; 58 89 C3 C1 FB 1F 31 D8 29 D8 25 FF 03 00 00 0F BF 84 47 78 05 00 00 C1 F8 05 50 (FASM Re-Assemble)

db "cos", 0, 0, 0, 0, 0
dw FUNC80_END - FUNC80_START

FUNC80_START:
pop    eax
mov    ebx,eax
sar    ebx,0x1f
xor    eax,ebx
sub    eax,ebx
and    eax,0x3ff
movsx  eax,WORD [edi+eax*2+0x578]
sar    eax,0x5
push   eax 
FUNC80_END:

; =============================================================================
; Syntax:      tan(x)
; Description: Bounded tangent function of x, where x is an integer between -256 and 256, inclusive, and the value returned is an integer between -512 and 512, inclusive (Windows) or -1024 and 1024, inclusive (Mac OS)
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 60 bytes)
; =============================================================================
; 58 8B D8 0B DB 7D 02 F7 DB 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 2D 00 01 00 00 79 02 F7 D8 25 FF 03 00 00 0F BF 84 47 78 05 00 00 0B DB 75 04 33 C0 EB 06 C1 E0 0A 99 F7 FB 50 (3.0.4 Original ASM)
; 58 89 C3 09 DB 7D 02 F7 DB 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 2D 00 01 00 00 79 02 F7 D8 25 FF 03 00 00 0F BF 84 47 78 05 00 00 09 DB 75 04 31 C0 EB 06 C1 E0 0A 99 F7 FB 50 (FASM Re-Assemble)

db "tan", 0, 0, 0, 0, 0
dw FUNC81_END - FUNC81_START

FUNC81_START:
pop    eax
mov    ebx,eax
or     ebx,ebx
jge    FUNC81_START+0x9
neg    ebx
and    ebx,0x3ff
movsx  ebx,WORD [edi+ebx*2+0x578]
sub    eax,0x100
jns    FUNC81_START+0x20
neg    eax
and    eax,0x3ff
movsx  eax,WORD [edi+eax*2+0x578]
or     ebx,ebx
jne    FUNC81_START+0x35
xor    eax,eax
jmp    FUNC81_START+0x3b
shl    eax,0xa
cdq
idiv   ebx
push   eax 
FUNC81_END:

; =============================================================================
; Syntax:      get(i)
; Description: Returns the current cell value at i
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 20 bytes)
; =============================================================================
; 58 0B C0 7C 0E 3D FF 00 00 00 7F 07 8B 84 87 78 35 00 00 50 (3.0.4 Original ASM)
; 58 09 C0 7C 0E 3D FF 00 00 00 7F 07 8B 84 87 78 35 00 00 50 (FASM Re-Assemble)

db "get", 0, 0, 0, 0, 0
dw FUNC82_END - FUNC82_START

FUNC82_START:
pop    eax
or     eax,eax
jl     FUNC82_START+0x13
cmp    eax,0xff
jg     FUNC82_START+0x13
mov    eax,DWORD [edi+eax*4+0x3578]
push   eax 
FUNC82_END:

; =============================================================================
; Syntax:      put(v,i)
; Description: Puts the new value v into cell i
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 21 bytes)
; =============================================================================
; 58 5B 0B C0 7C 0E 3D FF 00 00 00 7F 07 89 9C 87 78 35 00 00 53 (3.0.4 Original ASM)
; 58 5B 09 C0 7C 0E 3D FF 00 00 00 7F 07 89 9C 87 78 35 00 00 53 (FASM Re-Assemble)

db "put", 0, 0, 0, 0, 0
dw FUNC83_END - FUNC83_START

FUNC83_START:
pop    eax
pop    ebx
or     eax,eax
jl     FUNC83_START+0x14
cmp    eax,0xff
jg     FUNC83_START+0x14
mov    DWORD [edi+eax*4+0x3578],ebx
push   ebx 
FUNC83_END:

; =============================================================================

db 0, 0, 0, 0, 0, 0, 0, 0
dw 0
