program pow_v1;

uses
  SysUtils;

const
  funcname = 'pow';
  paramnum = 2;
  version  = '1 (borland variant)';
  author   = 'Daniel Marschall';

const
  linewidth = 20;

{$APPTYPE CONSOLE}
{$OPTIMIZATION OFF}

var
  i: PByte;
  j: pointer;
  k, z: smallint;
  s: string;

// High performance variant from Borland
// Without Floating instructions
// ASM code manually (size) optimized

// Source: Math.pas
// Copyright (c) 1996, 2001 Borland Software Corporation
// Modified: Extended->Integer, DivZero prevention (output 0 instead)
// Invariant: Y >= 0 & Result*X**Y = X**I.  Init Y = I and Result = 1.
function IntPower(X: Integer; I: Integer): Integer;
var
  Y: Integer;
begin
  Y := Abs(I);
  Result := 1;
  while Y > 0 do begin
    while not Odd(Y) do
    begin
      Y := Y shr 1;
      X := X * X
    end;
    Dec(Y);
    Result := Result * X
  end;
  // if I < 0 then Result := 1 div Result;
  if (I < 0) and (abs(Result) <> 1) then Result := 0;
end;

function pow(param_base, param_exp: integer): integer; stdcall;
asm
// Mssen erhalten bleiben:
push EDI
push ESI
push ESP
push EBP
push EBX

// Filter Factory: Parameterreihenfolge links nach rechts
push param_base
push param_exp

// jmp @@__print

// Kann keinen <<MARKER>> machen, da <</MARKER>> ein JMP davor bentigt
// (da POW ausgefhrt werden kann)
@@__marker__begin:

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

pop esi {param_exp}
pop ecx {param_base}
mov eax,esi	       	// .
cdq			// .
xor eax,edx		// .
sub eax,edx		// y = abs(param_exp);
xor edx,edx		// .
inc edx			// result = 1;
jmp @@pow_3	        // goto @@pow_3;
@@pow_1:		// @@pow_1:
shr eax,1		// y = y >> 1;
mov ebx,ecx		// .
imul ebx,ecx		// .
mov ecx,ebx		// param_base = param_base * param_base;
@@pow_2:		// @@pow_2:
test al,$01		// .
jz @@pow_1	        // if (y & 1 == 0) goto @@pow_1; // not odd
dec eax			// y--;
imul edx,ecx		// result = result * param_base;
@@pow_3:		// @@pow_3:
test eax,eax		// .
jnle @@pow_2		// if (y > 0) goto @@pow_2;
test esi,esi		// .
jnl @@pow_4	        // if (param_exp >= 0) goto @@pow_4; // return result;
cmp edx,$01		// .
jz @@pow_4		// if (result == 1) goto @@pow_4; // return result;
cmp edx,-$01		// .
jz @@pow_4	        // if (result == -1) goto @@pow_4; // return result;
xor edx,edx		// result = 0;
@@pow_4:		// @@pow_4:
push edx		// return result;

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

@@__marker__end:

@@__print:
lea eax,[@@__marker__begin]
mov i,eax
lea eax,[@@__marker__end]
mov j,eax

// Result der FF-Function
pop eax
mov result,eax

// Wiederherstellen
pop EBX
pop EBP
pop ESP
pop ESI
pop EDI
end;

begin
  pow(0,0); // to get i and j

  k := integer(j)-integer(i);
  WriteLn('Function: ' + funcname);
  WriteLn('Version:  ' + version);
  WriteLn('Params:   ' + IntToStr(paramnum));
  WriteLn('Compiled: ' + DateTimeToStr(FileDateToDateTime(FileAge(paramstr(0)))));
  WriteLn('Author:   ' + author);
  WriteLn('Length:   ' + inttostr(k));

  WriteLn('');
  WriteLn('Signature (FUNC):');
  for z := 1 to length(funcname) do
  begin
    Write(inttohex(ord(funcname[z]), 2) + ' ');
  end;
  for z := 8 downto length(funcname)+1 do
  begin
    Write('00 ');
  end;
  Write(inttohex(Lo(paramnum), 2)+' ');
  WriteLn(inttohex(Hi(paramnum), 2));

  WriteLn('');
  WriteLn('Prefix (OPER):');
  for z := 1 to length(funcname) do
  begin
    Write(inttohex(ord(funcname[z]), 2) + ' ');
  end;
  for z := 8 downto length(funcname)+1 do
  begin
    Write('00 ');
  end;
  Write(inttohex(Lo(k), 2)+' ');
  WriteLn(inttohex(Hi(k), 2));

  WriteLn('');
  WriteLn('Code (OPER):');
  for z := 0 to k-1 do
  begin
    Write(inttohex(ord(i^), 2));
    if (z = k-1) or ((z+1) mod linewidth = 0) then WriteLn('') else Write(' ');
    i := pointer(integer(i) + 1);
  end;

  WriteLn('');
  WriteLn('Self Tests:');
  WriteLn('pow( 5,  6) = ' + inttostr(pow(5,6)));
  WriteLn('pow( 0,  0) = ' + inttostr(pow(0,0)));
  WriteLn('pow(-1,  3) = ' + inttostr(pow(-1,3)));
  WriteLn('pow( 3, -1) = ' + inttostr(pow(3,-1)));
  WriteLn('pow( 1, -1) = ' + inttostr(pow(1,-1)));

  if paramstr(1) <> 'nowait' then
  begin
    ReadLn(s);
  end;
end.
