program cnv_v3;

uses
  SysUtils;

const
  funcname = 'cnv';
  paramnum = 10;
  version  = '3 (neighbor pixel variant)';
  author   = 'Daniel Marschall';

const
  linewidth = 20;
  
{$APPTYPE CONSOLE}
{$OPTIMIZATION OFF}

var
  i: PByte;
  j: pointer;
  k, z: smallint;
  s: string;

begin
  asm
    jmp @@__print
    db '<<MARKER>>';
    @@__marker__begin:

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

(*

+-------+---------------------+
|       | x>0           x<X-1 |
+-------+---------------------+
| y>0   | f11    f12    f13   |
|       | f21    f22    f23   |
| y<Y-1 | f31    f32    f33   |
+-------+---------------------+

*)

{ Prolog }
mov eax,[esp] {param_d}
or eax,eax
jz @cnv_div_zero
mov ecx,[edi+$000045a8] {CANVAS_OFFSET}
mov eax,[edi+$58] {z}
sub eax,[edi+$5c] {zmin}
add ecx,eax
mov eax,[edi+$4c] {y}
dec eax
sub eax,[edi+$50] {ymin}
mul dword ptr [edi+$000045a0] {ROW_SIZE}
add ecx,eax
mov eax,[edi+$40] {x}
dec eax
sub eax,[edi+$44] {xmin}
mov ebx,[edi+$60] {Z}
sub ebx,[edi+$5c] {zmin}
mul ebx
add ecx,eax
xor eax,eax // Optimized for better performance. Was "sub eax,eax"
mov edx,ecx // begin new line (-> f11)

{ Field f11 }
push edx // backup position
mov esi,[edi+$4c] {y}
or esi,esi
jnle @cnv_f11_y_ok // if (y>0) jmp @cnv_f11_y_ok
        // y=0
        // Correct y value.
        add edx,[edi+$000045a0] {ROW_SIZE} // Next line (-> f21)
@cnv_f11_y_ok:
        mov esi,[edi+$40] {x}
        or esi,esi
        jnle @cnv_f11_yx_ok // if (x>0) jmp @cnv_f11_yx_ok

	// x=0
        // Correct x value.
	add edx,ebx // Next pixel (-> f22 if y=0, else f12)
@cnv_f11_yx_ok:
movzx esi,[edx]
pop edx // restore position
imul esi,[esp+$24] {param_m11}
add eax,esi
add edx,ebx // Next pixel (-> f12)

{ Field f12 }
push edx // backup position
mov esi,[edi+$4c] {y}
or esi,esi
jnle @cnv_f12_yx_ok // if (y>0) jmp @cnv_f12_yx_ok
        // y=0
        // Correct y value.
        add edx,[edi+$000045a0] {ROW_SIZE} // Next line (-> f22)
@cnv_f12_yx_ok:
movzx esi,[edx]
pop edx // restore position
imul esi,[esp+$20] {param_m12}
add eax,esi
add edx,ebx // Next pixel (-> f13)

{ Field f13 }
push edx // backup position
mov esi,[edi+$4c] {y}
or esi,esi
jnle @cnv_f13_y_ok // if (y>0) jmp @cnv_f13_y_ok
        // y=0
        // Correct y value.
        add edx,[edi+$000045a0] {ROW_SIZE} // Next line (-> f23)
@cnv_f13_y_ok:
        mov esi,[edi+$48] {X}
        dec esi
        cmp esi,[edi+$40] {x}
        jnle @cnv_f13_yx_ok // if (x<X-1) jmp @cnv_f13_yx_ok

	// x=X-1
        // Correct x value.
	sub edx,ebx // Prev pixel (-> f22 if y=0, else f12)
@cnv_f13_yx_ok:
movzx esi,[edx]
pop edx // restore position
imul esi,[esp+$1c] {param_m13}
add eax,esi
add ecx,[edi+$000045a0]	{ROW_SIZE} // begin new line
mov edx,ecx // Next line (-> f21)

{ Field f21 }
push edx // backup position
mov esi,[edi+$40] {x}
or esi,esi
jnle @cnv_f21_yx_ok // if (x>0) jmp @cnv_f21_yx_ok
	// x=0
        // Correct x value.
	add edx,ebx // Next pixel (-> f22)
@cnv_f21_yx_ok:
movzx esi,[edx]
pop edx // restore position
imul esi,[esp+$18] {param_m21}
add eax,esi
add edx,ebx // Next pixel (-> f22)

{ Field f22 }
movzx esi,[edx]
imul esi,[esp+$14] {param_m22}
add eax,esi
add edx,ebx // Next pixel (-> f23)

{ Field f23 }
push edx // backup position
mov esi,[edi+$48] {X}
dec esi
cmp esi,[edi+$40] {x}
jnle @cnv_f23_yx_ok // if (x<X-1) jmp @cnv_f23_yx_ok
	// x=X-1
        // Correct x value.
	sub edx,ebx // Prev pixel (-> f22)
@cnv_f23_yx_ok:
movzx esi,[edx]
pop edx // restore position
imul esi,[esp+$10] {param_m23}
add eax,esi
add ecx,[edi+$000045a0]	{ROW_SIZE} // begin new line
mov edx,ecx // Next line (-> f31)

{ Field f31 }
push edx // backup position
mov esi,[edi+$54] {Y}
dec esi
cmp esi,[edi+$4c] {y}
jnle @cnv_f31_y_ok // if (y<Y-1) jmp @cnv_f31_y_ok
        // y=Y-1
        // Correct y value.
        sub edx,[edi+$000045a0] {ROW_SIZE} // Prev line (-> f21)
@cnv_f31_y_ok:
        mov esi,[edi+$40] {x}
        or esi,esi
        jnle @cnv_f31_yx_ok // if (x>0) jmp @cnv_f31_yx_ok

	// x=0
        // Correct x value.
	add edx,ebx // Next pixel (-> f22 if y=Y-1, else f32)
@cnv_f31_yx_ok:
movzx esi,[edx]
pop edx // restore position
imul esi,[esp+$0c] {param_m31}
add eax,esi
add edx,ebx // Next pixel (-> f32)

{ Field f32 }
push edx // backup position
mov esi,[edi+$54] {Y}
dec esi
cmp esi,[edi+$4c] {y}
jnle @cnv_f32_yx_ok // if (y<Y-1) jmp @cnv_f32_yx_ok
	// y=Y-1
        // Correct y value.
        sub edx,[edi+$000045a0] {ROW_SIZE} // Prev line (-> f22)
@cnv_f32_yx_ok:
movzx esi,[edx]
pop edx // restore position
imul esi,[esp+$08] {param_m32}
add eax,esi
add edx,ebx // Next pixel (-> f33)

{ Field f33 }
push edx // backup position
mov esi,[edi+$54] {Y}
dec esi
cmp esi,[edi+$4c] {y}
jnle @cnv_f33_y_ok // if (y<Y-1) jmp @cnv_f33_y_ok
	// y=Y-1
        // Correct y value.
        sub edx,[edi+$000045a0] {ROW_SIZE} // Prev line (-> f23)
@cnv_f33_y_ok:
        mov esi,[edi+$48] {X}
        dec esi
        cmp esi,[edi+$40] {x}
        jnle @cnv_f33_yx_ok // if (x<X-1) jmp @cnv_f33_yx_ok

	// x=X-1
        // Correct x value.
	sub edx,ebx // Prev pixel (-> f22 if y=Y-1, else f32)
@cnv_f33_yx_ok:
movzx esi,[edx]
pop edx // restore position
imul esi,[esp+$04] {param_m33}
add eax,esi

{ Epilog }
cdq
idiv dword ptr [esp] {param_d}
@cnv_div_zero:
add esp,$28 // = 10x pop
push eax

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

    @@__marker__end:
    db '<</MARKER>>';

    @@__print:
    lea eax,[@@__marker__begin]
    mov i,eax
    lea eax,[@@__marker__end]
    mov j,eax
  end;

  k := integer(j)-integer(i);
  WriteLn('Function: ' + funcname);
  WriteLn('Version:  ' + version);
  WriteLn('Params:   ' + IntToStr(paramnum));
  WriteLn('Compiled: ' + DateTimeToStr(FileDateToDateTime(FileAge(paramstr(0)))));
  WriteLn('Author:   ' + author);
  WriteLn('Length:   ' + inttostr(k));

  WriteLn('');
  WriteLn('Signature (FUNC):');
  for z := 1 to length(funcname) do
  begin
    Write(inttohex(ord(funcname[z]), 2) + ' ');
  end;
  for z := 8 downto length(funcname)+1 do
  begin
    Write('00 ');
  end;
  Write(inttohex(Lo(paramnum), 2)+' ');
  WriteLn(inttohex(Hi(paramnum), 2));

  WriteLn('');
  WriteLn('Prefix (OPER):');
  for z := 1 to length(funcname) do
  begin
    Write(inttohex(ord(funcname[z]), 2) + ' ');
  end;
  for z := 8 downto length(funcname)+1 do
  begin
    Write('00 ');
  end;
  Write(inttohex(Lo(k), 2)+' ');
  WriteLn(inttohex(Hi(k), 2));

  WriteLn('');
  WriteLn('Code (OPER):');
  for z := 0 to k-1 do
  begin
    Write(inttohex(ord(i^), 2));
    if (z = k-1) or ((z+1) mod linewidth = 0) then WriteLn('') else Write(' ');
    i := pointer(integer(i) + 1);
  end;

  if paramstr(1) <> 'nowait' then
  begin
    ReadLn(s);
  end;
end.
