use32

; =============================================================================
; Syntax:      (reserved token name "error")
; Description: For internal use. Gets called in the case that a syntax error appeared.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 3 bytes)
;              3.1.0 "sub r,r" -> "xor r,r"
; =============================================================================
; 2B C0 C3 (3.0.4 Original ASM)
; 29 C0 C3 (FASM Re-Assemble)
; 31 C0 C3 (Sub to XOR)

db "error", 0, 0, 0
dw FUNC0_END - FUNC0_START

FUNC0_START:
xor    eax,eax
ret
FUNC0_END:

; =============================================================================
; Syntax:      (reserved token name "prolog")
; Description: For internal use. Gets called at the beginning of the compiled channel asm code.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 8 bytes)
; =============================================================================
; 55 56 57 54 8B 7C 24 14 (3.0.4 Original ASM)
; 55 56 57 54 8B 7C 24 14 (FASM Re-Assemble)

db "prolog", 0, 0
dw FUNC1_END - FUNC1_START

FUNC1_START:
push   ebp
push   esi
push   edi
push   esp
mov    edi,DWORD [esp+0x14] 
FUNC1_END:

; =============================================================================
; Syntax:      (reserved token name "epilog")
; Description: For internal use. Gets called at the end of the compiled channel asm code.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; 58 5C 5F 5E 5D C3 (3.0.4 Original ASM)
; 58 5C 5F 5E 5D C3 (FASM Re-Assemble)

db "epilog", 0, 0
dw FUNC2_END - FUNC2_START

FUNC2_START:
pop    eax
pop    esp
pop    edi
pop    esi
pop    ebp
ret 
FUNC2_END:

; =============================================================================
; Syntax:      (reserved token name "number")
; Description: For internal use. ASM-Prefix to every number. The number itself is +4 byte ASM size.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 1+4 bytes)
; =============================================================================
; 68 __ __ __ __ (3.0.4 Original ASM)
; 68 __ __ __ __ (FASM Re-Assemble)

db "number", 0, 0
dw FUNC3_END - FUNC3_START

FUNC3_START:
db 104 ; the next 4 bytes in this instruction will be added by the plugin!
FUNC3_END:

; =============================================================================
; Syntax:      a,b
; Description: Inside a function: Separates two expressions.
;              Outside a function: "Forget everything" and start over again.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 1 byte)
; =============================================================================
; 58 (3.0.4 Original ASM)
; 58 (FASM Re-Assemble)

db ",", 0, 0, 0, 0, 0, 0, 0
dw FUNC4_END - FUNC4_START

FUNC4_START:
pop    eax
FUNC4_END:

; =============================================================================
; Syntax:      a?b:c
; Description: If a then b else c
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 11 bytes)
;              3.1.0 Daniel Marschall: Size Optimization (size = 10 bytes)
; =============================================================================
; 58 5B 59 83 F9 00 74 02 8B C3 50 (3.0.4 Original ASM)
; 58 5B 59 83 F9 00 74 02 89 D8 50 (FASM Re-Assemble)
; 58 5B 59 09 C9 74 02 89 D8 50 (ASM Size Optimization)

db "?", 0, 0, 0, 0, 0, 0, 0
dw FUNC5_END - FUNC5_START

FUNC5_START:
pop    eax
pop    ebx
pop    ecx
or     ecx,ecx  ; cmp ecx,0x0
je     FUNC5_DONE
mov    eax,ebx
FUNC5_DONE:
push   eax 
FUNC5_END:

; =============================================================================
; Syntax:      !a
; Description: Logical "not"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 9 bytes)
; =============================================================================
; 58 83 F8 01 1B C0 F7 D8 50 (3.0.4 Original ASM)
; 58 83 F8 01 19 C0 F7 D8 50 (FASM Re-Assemble)

db "!", 0, 0, 0, 0, 0, 0, 0
dw FUNC6_END - FUNC6_START

FUNC6_START:
pop    eax
cmp    eax,0x1
sbb    eax,eax
neg    eax
push   eax 
FUNC6_END:

; =============================================================================
; Syntax:      a&&b
; Description: Logical "and"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 19 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 14 bytes)
; =============================================================================
; 58 5B 83 F8 00 B8 00 00 00 00 74 06 83 FB 00 74 01 40 50 (3.0.4 Original ASM)
; 58 5B 83 F8 00 B8 00 00 00 00 74 06 83 FB 00 74 01 40 50 (FASM Re-Assemble)
; 58 5B 09 C0 74 07 31 C0 09 DB 74 01 40 50 (ASM Size Optimization)

db "&&", 0, 0, 0, 0, 0, 0
dw FUNC7_END - FUNC7_START

FUNC7_START:
pop    eax  ; param b
pop    ebx  ; param a
or     eax,eax
jz     FUNC7_DONE
xor    eax,eax
or     ebx,ebx
je     FUNC7_DONE
inc    eax
FUNC7_DONE:
push   eax 
FUNC7_END:

; =============================================================================
; Syntax:      a||b
; Description: Logical "or"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 19 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 14 bytes)
; =============================================================================
; 58 5B 83 F8 00 B8 00 00 00 00 75 05 83 FB 00 74 01 40 50 (3.0.4 Original ASM)
; 58 5B 83 F8 00 B8 00 00 00 00 75 05 83 FB 00 74 01 40 50 (FASM Re-Assemble)
; 58 5B 09 C0 75 04 09 DB 74 03 31 C0 40 50 (ASM Size Optimization)

db "||", 0, 0, 0, 0, 0, 0
dw FUNC8_END - FUNC8_START

FUNC8_START:
pop    eax  ; param_b
pop    ebx  ; param_a
or     eax,eax
jnz    FUNC8_RETURN_1
or     ebx,ebx
jz     FUNC8_RETURN_0
FUNC8_RETURN_1:
xor    eax,eax
inc    eax
FUNC8_RETURN_0:
push   eax
FUNC8_END:

; =============================================================================
; Syntax:      ~a
; Description: Bitwise "not"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 58 F7 D0 50 (3.0.4 Original ASM)
; 58 F7 D0 50 (FASM Re-Assemble)

db "~", 0, 0, 0, 0, 0, 0, 0
dw FUNC9_END - FUNC9_START

FUNC9_START:
pop    eax
not    eax
push   eax 
FUNC9_END:

; =============================================================================
; Syntax:      a&b
; Description: Bitwise "and"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 58 5B 23 C3 50 (3.0.4 Original ASM)
; 58 5B 21 D8 50 (FASM Re-Assemble)

db "&", 0, 0, 0, 0, 0, 0, 0
dw FUNC10_END - FUNC10_START

FUNC10_START:
pop    eax
pop    ebx
and    eax,ebx
push   eax 
FUNC10_END:

; =============================================================================
; Syntax:      a|b
; Description: Bitwise "or"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 58 5B 0B C3 50 (3.0.4 Original ASM)
; 58 5B 09 D8 50 (FASM Re-Assemble)

db "|", 0, 0, 0, 0, 0, 0, 0
dw FUNC11_END - FUNC11_START

FUNC11_START:
pop    eax
pop    ebx
or     eax,ebx
push   eax 
FUNC11_END:

; =============================================================================
; Syntax:      a^b
; Description: Bitwise "xor"
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 58 5B 33 C3 50 (3.0.4 Original ASM)
; 58 5B 31 D8 50 (FASM Re-Assemble)

db "^", 0, 0, 0, 0, 0, 0, 0
dw FUNC12_END - FUNC12_START

FUNC12_START:
pop    eax
pop    ebx
xor    eax,ebx
push   eax 
FUNC12_END:

; =============================================================================
; Syntax:      a==b
; Description: Equal?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 10 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 94 C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 94 C0 50 (FASM Re-Assemble)
; 59 5B 31 C0 39 D9 0F 94 C0 50 (ASM Size Optimization)

db "==", 0, 0, 0, 0, 0, 0
dw FUNC13_END - FUNC13_START

FUNC13_START:
pop    ecx  ; param_b
pop    ebx  ; param_a
xor    eax,eax
cmp    ecx,ebx
sete   al
push   eax 
FUNC13_END:

; =============================================================================
; Syntax:      a!=b
; Description: Not equal?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 10 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 95 C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 95 C0 50 (FASM Re-Assemble)
; 59 5B 31 C0 39 D9 0F 95 C0 50 (ASM Size Optimization)

db "!=", 0, 0, 0, 0, 0, 0
dw FUNC14_END - FUNC14_START

FUNC14_START:
pop    ecx  ; param_b
pop    ebx  ; param_a
xor    eax,eax
cmp    ecx,ebx
setne  al
push   eax 
FUNC14_END:

; =============================================================================
; Syntax:      a<b
; Description: a smaller than b?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 10 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 9F C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 9F C0 50 (FASM Re-Assemble)
; 59 5B 31 C0 39 D9 0F 9F C0 50 (ASM Size Optimization)

db "<", 0, 0, 0, 0, 0, 0, 0
dw FUNC15_END - FUNC15_START

FUNC15_START:
pop    ecx  ; param_b
pop    ebx  ; param_a
xor    eax,eax
cmp    ecx,ebx
setg   al
push   eax 
FUNC15_END:

; =============================================================================
; Syntax:      a<=b
; Description: a smaller or equal to b?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 10 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 9D C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 9D C0 50 (FASM Re-Assemble)
; 59 5B 31 C0 39 D9 0F 9D C0 50 (ASM Size Optimization)

db "<=", 0, 0, 0, 0, 0, 0
dw FUNC16_END - FUNC16_START

FUNC16_START:
pop    ecx  ; param_b
pop    ebx  ; param_a
xor    eax,eax
cmp    ecx,ebx
setge  al
push   eax 
FUNC16_END:

; =============================================================================
; Syntax:      a>b
; Description: a greater than b?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 10 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 9C C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 9C C0 50 (FASM Re-Assemble)
; 59 5B 31 C0 39 D9 0F 9C C0 50 (ASM Size Optimization)

db ">", 0, 0, 0, 0, 0, 0, 0
dw FUNC17_END - FUNC17_START

FUNC17_START:
pop    ecx  ; param_b
pop    ebx  ; param_a
xor    eax,eax
cmp    ecx,ebx
setl   al
push   eax 
FUNC17_END:

; =============================================================================
; Syntax:      a>=b
; Description: a greater or equal b?
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 13 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 10 bytes)
; =============================================================================
; 58 5B 3B C3 B8 00 00 00 00 0F 9E C0 50 (3.0.4 Original ASM)
; 58 5B 39 D8 B8 00 00 00 00 0F 9E C0 50 (FASM Re-Assemble)
; 59 5B 31 C0 39 D9 0F 9E C0 50 (ASM Size Optimization)

db ">=", 0, 0, 0, 0, 0, 0
dw FUNC18_END - FUNC18_START

FUNC18_START:
pop    ecx  ; param_b
pop    ebx  ; param_a
xor    eax,eax
cmp    ecx,ebx
setle  al
push   eax 
FUNC18_END:

; =============================================================================
; Syntax:      a<<b
; Description: Left shift
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 59 58 D3 E0 50 (3.0.4 Original ASM)
; 59 58 D3 E0 50 (FASM Re-Assemble)

db "<<", 0, 0, 0, 0, 0, 0
dw FUNC19_END - FUNC19_START

FUNC19_START:
pop    ecx
pop    eax
shl    eax,cl
push   eax 
FUNC19_END:

; =============================================================================
; Syntax:      a>>b
; Description: Right shift
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 59 58 D3 E8 50 (3.0.4 Original ASM)
; 59 58 D3 E8 50 (FASM Re-Assemble)

db ">>", 0, 0, 0, 0, 0, 0
dw FUNC20_END - FUNC20_START

FUNC20_START:
pop    ecx
pop    eax
shr    eax,cl
push   eax 
FUNC20_END:

; =============================================================================
; Syntax:      (reserved token name "negate")
; Description: For internal use only. The real sign is "-a".
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 58 F7 D8 50 (3.0.4 Original ASM)
; 58 F7 D8 50 (FASM Re-Assemble)

db "negate", 0, 0
dw FUNC21_END - FUNC21_START

FUNC21_START:
pop    eax
neg    eax
push   eax 
FUNC21_END:

; =============================================================================
; Syntax:      a+b
; Description: Add
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 58 5B 03 C3 50 (3.0.4 Original ASM)
; 58 5B 01 D8 50 (FASM Re-Assemble)

db "+", 0, 0, 0, 0, 0, 0, 0
dw FUNC22_END - FUNC22_START

FUNC22_START:
pop    eax
pop    ebx
add    eax,ebx
push   eax 
FUNC22_END:

; =============================================================================
; Syntax:      a-b
; Description: Minus
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 5 bytes)
; =============================================================================
; 5B 58 2B C3 50 (3.0.4 Original ASM)
; 5B 58 29 D8 50 (FASM Re-Assemble)

db "-", 0, 0, 0, 0, 0, 0, 0
dw FUNC23_END - FUNC23_START

FUNC23_START:
pop    ebx
pop    eax
sub    eax,ebx
push   eax 
FUNC23_END:

; =============================================================================
; Syntax:      a*b
; Description: Multiply
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
; =============================================================================
; 58 5B 0F AF C3 50 (3.0.4 Original ASM)
; 58 5B 0F AF C3 50 (FASM Re-Assemble)

db "*", 0, 0, 0, 0, 0, 0, 0
dw FUNC24_END - FUNC24_START

FUNC24_START:
pop    eax
pop    ebx
imul   eax,ebx
push   eax 
FUNC24_END:

; =============================================================================
; Syntax:      a/b
; Description: Division
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 14 bytes)
;              3.1.0 "sub r,r" -> "xor r,r"
; =============================================================================
; 5B 58 0B DB 75 04 2B C0 EB 03 99 F7 FB 50 (3.0.4 Original ASM)
; 5B 58 09 DB 75 04 29 C0 EB 03 99 F7 FB 50 (FASM Re-Assemble)
; 5B 58 09 DB 75 04 31 C0 EB 03 99 F7 FB 50 (Sub to XOR)

db "/", 0, 0, 0, 0, 0, 0, 0
dw FUNC25_END - FUNC25_START

FUNC25_START:
pop    ebx
pop    eax
or     ebx,ebx
jne    FUNC25_1
xor    eax,eax
jmp    FUNC25_2
FUNC25_1:
cdq
idiv   ebx
FUNC25_2:
push   eax 
FUNC25_END:

; =============================================================================
; Syntax:      a%b
; Description: Modulo
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 14 bytes)
;              3.1.0 "sub r,r" -> "xor r,r"
; =============================================================================
; 5B 58 0B DB 75 04 2B D2 EB 03 99 F7 FB 52 (3.0.4 Original ASM)
; 5B 58 09 DB 75 04 29 D2 EB 03 99 F7 FB 52 (FASM Re-Assemble)
; 5B 58 09 DB 75 04 31 D2 EB 03 99 F7 FB 52 (Sub to XOR)

db "%", 0, 0, 0, 0, 0, 0, 0
dw FUNC26_END - FUNC26_START

FUNC26_START:
pop    ebx
pop    eax
or     ebx,ebx
jne    FUNC26_1
xor    edx,edx
jmp    FUNC26_2
FUNC26_1:
cdq
idiv   ebx
FUNC26_2:
push   edx 
FUNC26_END:

; =============================================================================
; Syntax:      r
; Description: Current pixel's red channel value
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 3 bytes)
; =============================================================================
; 8B 07 50 (3.0.4 Original ASM)
; 8B 07 50 (FASM Re-Assemble)

db "r", 0, 0, 0, 0, 0, 0, 0
dw FUNC27_END - FUNC27_START

FUNC27_START:
mov    eax,DWORD [edi]
push   eax 
FUNC27_END:

; =============================================================================
; Syntax:      g
; Description: Current pixel's green channel value
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 04 50 (3.0.4 Original ASM)
; 8B 47 04 50 (FASM Re-Assemble)

db "g", 0, 0, 0, 0, 0, 0, 0
dw FUNC28_END - FUNC28_START

FUNC28_START:
mov    eax,DWORD [edi+0x4]
push   eax 
FUNC28_END:

; =============================================================================
; Syntax:      b
; Description: Current pixel's blue channel value
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 08 50 (3.0.4 Original ASM)
; 8B 47 08 50 (FASM Re-Assemble)

db "b", 0, 0, 0, 0, 0, 0, 0
dw FUNC29_END - FUNC29_START

FUNC29_START:
mov    eax,DWORD [edi+0x8]
push   eax 
FUNC29_END:

; =============================================================================
; Syntax:      a
; Description: Current pixel's alpha channel value
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 0C 50 (3.0.4 Original ASM)
; 8B 47 0C 50 (FASM Re-Assemble)

db "a", 0, 0, 0, 0, 0, 0, 0
dw FUNC30_END - FUNC30_START

FUNC30_START:
mov    eax,DWORD [edi+0xc]
push   eax 
FUNC30_END:

; =============================================================================
; Syntax:      c
; Description: Value of the current channel, whichever channel the expression is defining
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 10 bytes)
; =============================================================================
; 8B 47 58 2B 47 5C 8B 04 87 50 (3.0.4 Original ASM)
; 8B 47 58 2B 47 5C 8B 04 87 50 (FASM Re-Assemble)

db "c", 0, 0, 0, 0, 0, 0, 0
dw FUNC31_END - FUNC31_START

FUNC31_START:
mov    eax,DWORD [edi+0x58]
sub    eax,DWORD [edi+0x5c]
mov    eax,DWORD [edi+eax*4]
push   eax 
FUNC31_END:

; =============================================================================
; Syntax:      i
; Description: Y value of the YUV color space
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 55 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization, completely rewritten (size = 37 bytes)
;              3.1.0 Daniel Marschall: New formula is now more accurate (size = 40 bytes)
; =============================================================================
; In Filter Factory 3.0.4:
; Y  = (76R + 150G + 29B) / 256
; Range = [0..254]
; =============================================================================
; Patch by Daniel Marschall:
; Y := 0.299R + 0.587G + 0.114B
; Y  = (299R + 587G + 114B) / 1000
; Range = [0..255]
; =============================================================================
; 8B 77 04 8D 34 B6 03 F6 8D 34 76 8D 34 B6 8B 4F 08 8B D1 C1 E1 03 2B CA 8D 0C 8A 03 F1 8B 07 8B C8 8D 04 C0 8D 04 41 8D 04 86 99 81 E2 FF 00 00 00 03 C2 C1 F8 08 50 (3.0.4 Original ASM)
; 8B 77 04 8D 34 B6 01 F6 8D 34 76 8D 34 B6 8B 4F 08 89 CA C1 E1 03 29 D1 8D 0C 8A 01 CE 8B 07 89 C1 8D 04 C0 8D 04 41 8D 04 86 99 81 E2 FF 00 00 00 01 D0 C1 F8 08 50 (FASM Re-Assemble)
; 31 C0 8B 37 69 F6 4C 00 00 00 01 F0 8B 77 04 69 F6 96 00 00 00 01 F0 8B 77 08 69 F6 1D 00 00 00 01 F0 B9 00 01 00 00 99 F7 F9 50 (ASM Size Optimization)
; 31 C0 8B 37 6B F6 4C          01 F0 8B 77 04 69 F6 96 00 00 00 01 F0 8B 77 08 6B F6 1D          01 F0 B9 00 01 00 00 99 F7 F9 50 (ASM Size Optimization FASM)
; 31 C0 8B 37 6B F6 4C          01 F0 8B 77 04 69 F6 96 00 00 00 01 F0 8B 77 08 6B F6 1D          01 F0 B9 FF 00 00 00 99 F7 F9 50 (YUV Bugfix)

db "i", 0, 0, 0, 0, 0, 0, 0
dw FUNC32_END - FUNC32_START

FUNC32_START:
xor eax,eax

mov esi,[edi] ;r
imul esi,esi,299
add eax,esi

mov esi,[edi+0x4] ;g
imul esi,esi,587
add eax,esi

mov esi,[edi+0x8] ;b
imul esi,esi,114
add eax,esi

mov ecx,1000
cdq
idiv ecx

push eax
FUNC32_END:

; =============================================================================
; Syntax:      u
; Description: U value of the YUV color space
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 51 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization, completely rewritten (size = 34 bytes)
;              3.1.0 Daniel Marschall: New formula is now more accurate (size = 43 bytes)
; =============================================================================
; In Filter Factory 3.0.4:
; U  = (-19R - 37G + 56B) / 256
; U  = (-0.074R - 0.144G + 0.218B)
; Range = [-55..55]
; =============================================================================
; Patch by Daniel Marschall:
; U := (B-Y) * 0.493
; U  = (B - (0.299R + 0.587G + 0.114B)) * 0.493
; U  = (B - 0.299R - 0.587G - 0.114B) * 0.493
; U  = (-0.299R - 0.587G + 0.886B) * 0.493
; U  = -0.147407R - 0.289391G + 0.436798B
; U  = (-147407R - 289391G + 436798B) / 1000000
; Range = [-111..111]
; Divide by 2 to become compatible with Filter Factory again...
; U  = (-147407R - 289391G + 436798B) / 2000000
; Range = [-55..55]
; =============================================================================
; 8B 77 04 8B C6 8D 34 F6 8D 34 B0 F7 DE 8B 07 8B C8 8D 04 C0 8D 04 41 2B F0 8B 47 08 8B C8 C1 E0 03 2B C1 8D 04 C6 99 81 E2 FF 00 00 00 03 C2 C1 F8 08 50 (3.0.4 Original ASM)
; 8B 77 04 89 F0 8D 34 F6 8D 34 B0 F7 DE 8B 07 89 C1 8D 04 C0 8D 04 41 29 C6 8B 47 08 89 C1 C1 E0 03 29 C8 8D 04 C6 99 81 E2 FF 00 00 00 01 D0 C1 F8 08 50 (FASM Re-Assemble)
; 31 C0 8B 37 69 F6 ED FF FF FF 01 F0 8B 77 04 69 F6 DB FF FF FF 01 F0 8B 77 08 69 F6 38 00 00 00 01 F0 B9 00 01 00 00 99 F7 F9 50 (ASM Size Optimization)
; 31 C0 8B 37 6B F6 ED          01 F0 8B 77 04 6B F6 DB          01 F0 8B 77 08 6B F6 38          01 F0 B9 00 01 00 00 99 F7 F9 50 (ASM Size Optimization FASM)
; 31 C0 8B 37 6B F6 ED          01 F0 8B 77 04 6B F6 DB          01 F0 8B 77 08 6B F6 38          01 F0 B9 FF 00 00 00 99 F7 F9 50 (YUV Bugfix)

db "u", 0, 0, 0, 0, 0, 0, 0
dw FUNC33_END - FUNC33_START

FUNC33_START:

xor eax,eax

mov esi,[edi] ;r
imul esi,esi,-147407
add eax,esi

mov esi,[edi+0x4] ;g
imul esi,esi,-289391
add eax,esi

mov esi,[edi+0x8] ;b
imul esi,esi,436798
add eax,esi

mov ecx,2000000
cdq
idiv ecx

push eax

FUNC33_END:

; =============================================================================
; Syntax:      v
; Description: V value of the YUV color space
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 39 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization, completely rewritten (size = 34 bytes)
;              3.1.0 Daniel Marschall: New formula is now more accurate (size = 43 bytes)
; =============================================================================
; In Filter Factory 3.0.4:
; V  = (78R - 65G - 13B) / 256
; V  = (0.304R - 0.253G - 0.050B)
; Range = [-77..77]
; =============================================================================
; Patch by Daniel Marschall:
; V := (R-Y) * 0.877
; V  = (R - (0.299R + 0.587G + 0.114B)) * 0.877
; V  = (R - 0.299R - 0.587G - 0.114B) * 0.877
; V  = (0.701R - 0.587G - 0.114B) * 0.877
; V  = (0.614777R - 0.514799G - 0.099978B)
; V  = (614777R - 514799G - 99978B) / 1000000
; Range = [-156..156]
; Divide by 2 to become compatible with Filter Factory again...
; V  = (614777R - 514799G - 99978B) / 2000000
; Range = [-78..78]
; =============================================================================
; 8B 17 8D 14 52 03 D2 8B 4F 04 8D 0C 89 2B D1 2B 57 08 8B C2 8D 14 52 8D 04 90 99 81 E2 FF 00 00 00 03 C2 C1 F8 08 50 (3.0.4 Original ASM)
; 8B 17 8D 14 52 01 D2 8B 4F 04 8D 0C 89 29 CA 2B 57 08 89 D0 8D 14 52 8D 04 90 99 81 E2 FF 00 00 00 01 D0 C1 F8 08 50 (FASM Re-Assemble)
; 31 C0 8B 37 6B F6 4E 01 F0 8B 77 04 6B F6 BF 01 F0 8B 77 08 6B F6 F3 01 F0 B9 00 01 00 00 99 F7 F9 50 (ASM Size Optimization)
; 31 C0 8B 37 6B F6 4E 01 F0 8B 77 04 6B F6 BF 01 F0 8B 77 08 6B F6 F3 01 F0 B9 FF 00 00 00 99 F7 F9 50 (YUV Bugfix)

db "v", 0, 0, 0, 0, 0, 0, 0
dw FUNC34_END - FUNC34_START

FUNC34_START:

xor eax,eax
 
mov esi,[edi] ;r
imul esi,esi,614777
add eax,esi
 
mov esi,[edi+0x4] ;g
imul esi,esi,-514799
add eax,esi
 
mov esi,[edi+0x8] ;b
imul esi,esi,-99978
add eax,esi
 
mov ecx,2000000
cdq
idiv ecx
 
push eax

FUNC34_END:

; =============================================================================
; Syntax:      d
; Description: Direction (angle) of the current pixel from the center of the image, where d is an integer between 0 and 1024, inclusive
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 117 bytes)
; =============================================================================
; 8B 47 54 8B 5F 48 2B 47 50 2B 5F 44 D1 E8 D1 EB F7 D8 F7 DB 33 C9 03 47 4C 79 05 F7 D8 80 C9 04 03 5F 40 79 05 F7 DB 80 C9 03 3B C3 7E 04 80 F1 01 93 0B C0 7E 1B C1 E0 0A 99 F7 F3 74 13 66 8B 84 47 76 0D 00 00 C1 E0 09 99 BB 40 24 03 00 F7 FB 66 0F BA E1 00 73 07 F7 D8 05 00 01 00 00 66 0F BA E1 01 73 05 05 00 01 00 00 66 0F BA E1 02 73 02 F7 D8 50 (3.0.4 Original ASM)
; 8B 47 54 8B 5F 48 2B 47 50 2B 5F 44 D1 E8 D1 EB F7 D8 F7 DB 31 C9 03 47 4C 79 05 F7 D8 80 C9 04 03 5F 40 79 05 F7 DB 80 C9 03 39 D8 7E 04 80 F1 01 93 09 C0 7E 1B C1 E0 0A 99 F7 F3 74 13 66 8B 84 47 76 0D 00 00 C1 E0 09 99 BB 40 24 03 00 F7 FB 66 0F BA E1 00 73 07 F7 D8 05 00 01 00 00 66 0F BA E1 01 73 05 05 00 01 00 00 66 0F BA E1 02 73 02 F7 D8 50 (FASM Re-Assemble)

db "d", 0, 0, 0, 0, 0, 0, 0
dw FUNC35_END - FUNC35_START

FUNC35_START:
mov    eax,DWORD [edi+0x54]
mov    ebx,DWORD [edi+0x48]
sub    eax,DWORD [edi+0x50]
sub    ebx,DWORD [edi+0x44]
shr    eax,1
shr    ebx,1
neg    eax
neg    ebx
xor    ecx,ecx
add    eax,DWORD [edi+0x4c]
jns    FUNC35_1
neg    eax
or     cl,0x4
FUNC35_1:
add    ebx,DWORD [edi+0x40]
jns    FUNC35_2
neg    ebx
or     cl,0x3
FUNC35_2:
cmp    eax,ebx
jle    FUNC35_3
xor    cl,0x1
xchg   ebx,eax
FUNC35_3:
or     eax,eax
jle    FUNC35_4
shl    eax,0xa
cdq
div    ebx
je     FUNC35_4
mov    ax,WORD [edi+eax*2+0xd76]
shl    eax,0x9
cdq
mov    ebx,0x32440
idiv   ebx
FUNC35_4:
bt     cx,0x0
jae    FUNC35_5
neg    eax
add    eax,0x100
FUNC35_5:
bt     cx,0x1
jae    FUNC35_6
add    eax,0x100
FUNC35_6:
bt     cx,0x2
jae    FUNC35_7
neg    eax
FUNC35_7:
push   eax 
FUNC35_END:

; =============================================================================
; Syntax:      m
; Description: Distance (magnitude) from the center of the image to the current pixel
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 77 bytes)
; =============================================================================
; 8B 47 44 8B 5F 50 2B 47 48 2B 5F 54 D1 F8 D1 FB 03 47 40 03 5F 4C 8B D0 C1 FA 1F 33 C2 2B C2 8B D3 C1 FA 1F 33 DA 2B DA 3B C3 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 03 C3 50 (3.0.4 Original ASM)
; 8B 47 44 8B 5F 50 2B 47 48 2B 5F 54 D1 F8 D1 FB 03 47 40 03 5F 4C 89 C2 C1 FA 1F 31 D0 29 D0 89 DA C1 FA 1F 31 D3 29 D3 39 D8 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 01 D8 50 (FASM Re-Assemble)

db "m", 0, 0, 0, 0, 0, 0, 0
dw FUNC36_END - FUNC36_START

FUNC36_START:
mov    eax,DWORD [edi+0x44]
mov    ebx,DWORD [edi+0x50]
sub    eax,DWORD [edi+0x48]
sub    ebx,DWORD [edi+0x54]
sar    eax,1
sar    ebx,1
add    eax,DWORD [edi+0x40]
add    ebx,DWORD [edi+0x4c]
mov    edx,eax
sar    edx,0x1f
xor    eax,edx
sub    eax,edx
mov    edx,ebx
sar    edx,0x1f
xor    ebx,edx
sub    ebx,edx
cmp    eax,ebx
jne    FUNC36_1
mov    eax,0x6a0a
jmp    FUNC36_3
FUNC36_1:
jle    FUNC36_2
xchg   ebx,eax
FUNC36_2:
shl    eax,0xa
cdq
div    ebx
movzx  eax,WORD [edi+eax*2+0x1578]
FUNC36_3:
mul    ebx
shrd   eax,edx,0x10
add    eax,ebx
push   eax 
FUNC36_END:

; =============================================================================
; Syntax:      R
; Description: Max channel value for r; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 5 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)
; 68 FF 00 00 00 (ASM Size Optimization)

db "R", 0, 0, 0, 0, 0, 0, 0
dw FUNC37_END - FUNC37_START

FUNC37_START:
push   0xff
FUNC37_END:

; =============================================================================
; Syntax:      G
; Description: Max channel value for g; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 5 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)
; 68 FF 00 00 00 (ASM Size Optimization)

db "G", 0, 0, 0, 0, 0, 0, 0
dw FUNC38_END - FUNC38_START

FUNC38_START:
push   0xff
FUNC38_END:

; =============================================================================
; Syntax:      B
; Description: Max channel value for b; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 5 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)
; 68 FF 00 00 00 (ASM Size Optimization)

db "B", 0, 0, 0, 0, 0, 0, 0
dw FUNC39_END - FUNC39_START

FUNC39_START:
push   0xff
FUNC39_END:

; =============================================================================
; Syntax:      A
; Description: Max channel value for a; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 5 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)
; 68 FF 00 00 00 (ASM Size Optimization)

db "A", 0, 0, 0, 0, 0, 0, 0
dw FUNC40_END - FUNC40_START

FUNC40_START:
push   0xff
FUNC40_END:

; =============================================================================
; Syntax:      I
; Description: Max channel value for i; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 5 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)
; 68 FF 00 00 00 (ASM Size Optimization)

db "I", 0, 0, 0, 0, 0, 0, 0
dw FUNC41_END - FUNC41_START

FUNC41_START:
push   0xff
FUNC41_END:

; =============================================================================
; Syntax:      U
; Description: Max channel value for u; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 5 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)
; 68 FF 00 00 00 (ASM Size Optimization)

db "U", 0, 0, 0, 0, 0, 0, 0
dw FUNC42_END - FUNC42_START

FUNC42_START:
push   0xff
FUNC42_END:

; =============================================================================
; Syntax:      V
; Description: Max channel value for v; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 5 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)
; 68 FF 00 00 00 (ASM Size Optimization)

db "V", 0, 0, 0, 0, 0, 0, 0
dw FUNC43_END - FUNC43_START

FUNC43_START:
push   0xff
FUNC43_END:

; =============================================================================
; Syntax:      C
; Description: Max channel value for c; always 255
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 5 bytes)
; =============================================================================
; B8 FF 00 00 00 50 (3.0.4 Original ASM)
; B8 FF 00 00 00 50 (FASM Re-Assemble)
; 68 FF 00 00 00 (ASM Size Optimization)

db "C", 0, 0, 0, 0, 0, 0, 0
dw FUNC44_END - FUNC44_START

FUNC44_START:
push   0xff
FUNC44_END:

; =============================================================================
; Syntax:      D
; Description: Range of angles within the image, where dmin is always 0 and D is always 1024
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 5 bytes)
; =============================================================================
; B8 00 04 00 00 50 (3.0.4 Original ASM)
; B8 00 04 00 00 50 (FASM Re-Assemble)
; 68 00 04 00 00 (ASM Size Optimization)

db "D", 0, 0, 0, 0, 0, 0, 0
dw FUNC45_END - FUNC45_START

FUNC45_START:
push   1024
FUNC45_END:

; =============================================================================
; Syntax:      M
; Description: Range of magnitudes with the image, where mmin is always 0 and M is always one half the diagonal size of the image
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 53 bytes)
; =============================================================================
; 8B 47 48 8B 5F 54 2B 47 44 2B 5F 50 D1 F8 D1 FB 3B C3 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 03 C3 50 (3.0.4 Original ASM)
; 8B 47 48 8B 5F 54 2B 47 44 2B 5F 50 D1 F8 D1 FB 39 D8 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 01 D8 50 (FASM Re-Assemble)

db "M", 0, 0, 0, 0, 0, 0, 0
dw FUNC46_END - FUNC46_START

FUNC46_START:
mov    eax,DWORD [edi+0x48]
mov    ebx,DWORD [edi+0x54]
sub    eax,DWORD [edi+0x44]
sub    ebx,DWORD [edi+0x50]
sar    eax,1
sar    ebx,1
cmp    eax,ebx
jne    FUNC46_1
mov    eax,0x6a0a
jmp    FUNC46_3
FUNC46_1:
jle    FUNC46_2
xchg   ebx,eax
FUNC46_2:
shl    eax,0xa
cdq
div    ebx
movzx  eax,WORD [edi+eax*2+0x1578]
FUNC46_3:
mul    ebx
shrd   eax,edx,0x10
add    eax,ebx
push   eax 
FUNC46_END:

; =============================================================================
; Syntax:      x
; Description: x coordinate of the current pixel
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 40 50 (3.0.4 Original ASM)
; 8B 47 40 50 (FASM Re-Assemble)

db "x", 0, 0, 0, 0, 0, 0, 0
dw FUNC47_END - FUNC47_START

FUNC47_START:
mov    eax,DWORD [edi+0x40]
push   eax 
FUNC47_END:

; =============================================================================
; Syntax:      y
; Description: y coordinate of the current pixel
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 4C 50 (3.0.4 Original ASM)
; 8B 47 4C 50 (FASM Re-Assemble)

db "y", 0, 0, 0, 0, 0, 0, 0
dw FUNC48_END - FUNC48_START

FUNC48_START:
mov    eax,DWORD [edi+0x4c]
push   eax 
FUNC48_END:

; =============================================================================
; Syntax:      z
; Description: Channel index for the current expression
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 58 50 (3.0.4 Original ASM)
; 8B 47 58 50 (FASM Re-Assemble)

db "z", 0, 0, 0, 0, 0, 0, 0
dw FUNC49_END - FUNC49_START

FUNC49_START:
mov    eax,DWORD [edi+0x58]
push   eax 
FUNC49_END:

; =============================================================================
; Syntax:      X
; Description: Range of horizontal coordinates over the width of the image
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 48 50 (3.0.4 Original ASM)
; 8B 47 48 50 (FASM Re-Assemble)

db "X", 0, 0, 0, 0, 0, 0, 0
dw FUNC50_END - FUNC50_START

FUNC50_START:
mov    eax,DWORD [edi+0x48]
push   eax 
FUNC50_END:

; =============================================================================
; Syntax:      Y
; Description: Range of vertical coordinates over the height of the image
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 54 50 (3.0.4 Original ASM)
; 8B 47 54 50 (FASM Re-Assemble)

db "Y", 0, 0, 0, 0, 0, 0, 0
dw FUNC51_END - FUNC51_START

FUNC51_START:
mov    eax,DWORD [edi+0x54]
push   eax 
FUNC51_END:

; =============================================================================
; Syntax:      Z
; Description: Range of channel indexes within one pixel
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 60 50 (3.0.4 Original ASM)
; 8B 47 60 50 (FASM Re-Assemble)

db "Z", 0, 0, 0, 0, 0, 0, 0
dw FUNC52_END - FUNC52_START

FUNC52_START:
mov    eax,DWORD [edi+0x60]
push   eax 
FUNC52_END:

; =============================================================================
; Syntax:      xmin
; Description: Smallest x coodinate; always 0
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 44 50 (3.0.4 Original ASM)
; 8B 47 44 50 (FASM Re-Assemble)

db "xmin", 0, 0, 0, 0
dw FUNC53_END - FUNC53_START

FUNC53_START:
mov    eax,DWORD [edi+0x44]
push   eax 
FUNC53_END:

; =============================================================================
; Syntax:      ymin
; Description: Smallest y coodinate; always 0
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 50 50 (3.0.4 Original ASM)
; 8B 47 50 50 (FASM Re-Assemble)

db "ymin", 0, 0, 0, 0
dw FUNC54_END - FUNC54_START

FUNC54_START:
mov    eax,DWORD [edi+0x50]
push   eax 
FUNC54_END:

; =============================================================================
; Syntax:      zmin
; Description: Smallest z coodinate; always 0
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 4 bytes)
; =============================================================================
; 8B 47 5C 50 (3.0.4 Original ASM)
; 8B 47 5C 50 (FASM Re-Assemble)

db "zmin", 0, 0, 0, 0
dw FUNC55_END - FUNC55_START

FUNC55_START:
mov    eax,DWORD [edi+0x5c]
push   eax 
FUNC55_END:

; =============================================================================
; Syntax:      rmin
; Description: Smallest red value; always 0
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 3 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 2 bytes)
; =============================================================================
; 2B C0 50 (3.0.4 Original ASM)
; 29 C0 50 (FASM Re-Assemble)
; 6A 00 (ASM Size Optimization)

db "rmin", 0, 0, 0, 0
dw FUNC56_END - FUNC56_START

FUNC56_START:
push   0x0
FUNC56_END:

; =============================================================================
; Syntax:      tmax
; Description: Total number of frames (used only in Adobe Premiere). always returns 1. 
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 6 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 2 bytes)
; =============================================================================
; B8 01 00 00 00 50 (3.0.4 Original ASM)
; B8 01 00 00 00 50 (FASM Re-Assemble)
; 6A 01 (ASM Size Optimization)

db "tmax", 0, 0, 0, 0
dw FUNC57_END - FUNC57_START

FUNC57_START:
push   0x1
FUNC57_END:

; =============================================================================
; Syntax:      min(a,b)
; Description: Lesser of a and b
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 9 bytes)
; =============================================================================
; 58 59 3B C1 7C 02 8B C1 50 (3.0.4 Original ASM)
; 58 59 39 C8 7C 02 89 C8 50 (FASM Re-Assemble)

db "min", 0, 0, 0, 0, 0
dw FUNC58_END - FUNC58_START

FUNC58_START:
pop    eax
pop    ecx
cmp    eax,ecx
jl     FUNC58_DONE
mov    eax,ecx
FUNC58_DONE:
push   eax 
FUNC58_END:

; =============================================================================
; Syntax:      max(a,b)
; Description: Greater of a and b
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 9 bytes)
; =============================================================================
; 58 59 3B C1 7F 02 8B C1 50 (3.0.4 Original ASM)
; 58 59 39 C8 7F 02 89 C8 50 (FASM Re-Assemble)

db "max", 0, 0, 0, 0, 0
dw FUNC59_END - FUNC59_START

FUNC59_START:
pop    eax
pop    ecx
cmp    eax,ecx
jg     FUNC59_DONE
mov    eax,ecx
FUNC59_DONE:
push   eax 
FUNC59_END:

; =============================================================================
; Syntax:      abs(a)
; Description: Absolute value of a.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 7 bytes)
; =============================================================================
; 58 99 33 C2 2B C2 50 (3.0.4 Original ASM)
; 58 99 31 D0 29 D0 50 (FASM Re-Assemble)

db "abs", 0, 0, 0, 0, 0
dw FUNC60_END - FUNC60_START

FUNC60_START:
pop    eax
cdq
xor    eax,edx
sub    eax,edx
push   eax 
FUNC60_END:

; =============================================================================
; Syntax:      add(a,b,c)
; Description: Sum of a and b, or c, whichever is lesser
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 12 bytes)
; =============================================================================
; 59 5B 58 03 C3 3B C1 7C 02 8B C1 50 (3.0.4 Original ASM)
; 59 5B 58 01 D8 39 C8 7C 02 89 C8 50 (FASM Re-Assemble)

db "add", 0, 0, 0, 0, 0
dw FUNC61_END - FUNC61_START

FUNC61_START:
pop    ecx
pop    ebx
pop    eax
add    eax,ebx
cmp    eax,ecx
jl     FUNC61_DONE
mov    eax,ecx
FUNC61_DONE:
push   eax 
FUNC61_END:

; =============================================================================
; Syntax:      sub(a,b,c)
; Description: Difference of a and b, or c, whichever is greater
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 12 bytes)
; =============================================================================
; 59 5B 58 2B C3 3B C1 7F 02 8B C1 50 (3.0.4 Original ASM)
; 59 5B 58 29 D8 39 C8 7F 02 89 C8 50 (FASM Re-Assemble)

db "sub", 0, 0, 0, 0, 0
dw FUNC62_END - FUNC62_START

FUNC62_START:
pop    ecx
pop    ebx
pop    eax
sub    eax,ebx
cmp    eax,ecx
jg     FUNC62_DONE
mov    eax,ecx
FUNC62_DONE:
push   eax 
FUNC62_END:

; =============================================================================
; Syntax:      dif(a,b)
; Description: Absolute value of the difference of a and b
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 10 bytes)
; =============================================================================
; 58 5B 2B C3 99 33 C2 2B C2 50 (3.0.4 Original ASM)
; 58 5B 29 D8 99 31 D0 29 D0 50 (FASM Re-Assemble)

db "dif", 0, 0, 0, 0, 0
dw FUNC63_END - FUNC63_START

FUNC63_START:
pop    eax
pop    ebx
sub    eax,ebx
cdq
xor    eax,edx
sub    eax,edx
push   eax 
FUNC63_END:

; =============================================================================
; Syntax:      mix(a,b,n,d)
; Description: Mixture of a and b by fraction n/d, a*n/d+b*(d-n)/d
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 32 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimizaton (size = 29 bytes)
; =============================================================================
; 5B 59 5A 5E 0B DB B8 00 00 00 00 74 12 8B C3 2B C1 F7 EA F7 FB 8B E8 8B C1 F7 EE F7 FB 03 C5 50 (3.0.4 Original ASM)
; 5B 59 5A 5E 09 DB B8 00 00 00 00 74 12 89 D8 29 C8 F7 EA F7 FB 89 C5 89 C8 F7 EE F7 FB 01 E8 50 (FASM Re-Assemble)
; 5B 59 5A 5E 31 C0 0B DB 74 12 8B C3 2B C1 F7 EA F7 FB 8B E8 8B C1 F7 EE F7 FB 03 C5 50 (ASM Size Optimization)
; 5B 59 5A 5E 31 C0 09 DB 74 12 89 D8 29 C8 F7 EA F7 FB 89 C5 89 C8 F7 EE F7 FB 01 E8 50 (ASM Size Optimization FASM)

db "mix", 0, 0, 0, 0, 0
dw FUNC64_END - FUNC64_START

FUNC64_START:
pop    ebx
pop    ecx
pop    edx
pop    esi
xor    eax,eax      ; was: or ebx,ebx
or     ebx,ebx      ; was: mov eax,0x0
je     FUNC64_DONE
mov    eax,ebx
sub    eax,ecx
imul   edx
idiv   ebx
mov    ebp,eax
mov    eax,ecx
imul   esi
idiv   ebx
add    eax,ebp
FUNC64_DONE:
push   eax 
FUNC64_END:

; =============================================================================
; Syntax:      scl(a,il,ih,ol,oh)
; Description: Scale a from input range (il to ih) to output range (ol to oh)
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 39 bytes)
; =============================================================================
; 8B 6C 24 08 8B 4C 24 0C 2B E9 8B C5 74 15 8B 04 24 8B 74 24 04 2B C6 8B 54 24 10 2B D1 F7 EA F7 FD 03 C6 83 C4 14 50 (3.0.4 Original ASM)
; 8B 6C 24 08 8B 4C 24 0C 29 CD 89 E8 74 15 8B 04 24 8B 74 24 04 29 F0 8B 54 24 10 29 CA F7 EA F7 FD 01 F0 83 C4 14 50 (FASM Re-Assemble)

db "scl", 0, 0, 0, 0, 0
dw FUNC65_END - FUNC65_START

FUNC65_START:
mov    ebp,DWORD [esp+0x8]
mov    ecx,DWORD [esp+0xc]
sub    ebp,ecx
mov    eax,ebp
je     FUNC65_DONE
mov    eax,DWORD [esp]
mov    esi,DWORD [esp+0x4]
sub    eax,esi
mov    edx,DWORD [esp+0x10]
sub    edx,ecx
imul   edx
idiv   ebp
add    eax,esi
FUNC65_DONE:
add    esp,0x14  ; 5x POP
push   eax 
FUNC65_END:

; =============================================================================
; Syntax:      sqr(x)
; Description: Square root of x
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 31 bytes)
; =============================================================================
; 5B 83 FB 01 7E 18 8B CB D1 EB B8 02 00 00 00 3B D8 7E 0B 8B C1 99 F7 F3 03 D8 D1 EB EB F1 53 (3.0.4 Original ASM)
; 5B 83 FB 01 7E 18 89 D9 D1 EB B8 02 00 00 00 39 C3 7E 0B 89 C8 99 F7 F3 01 C3 D1 EB EB F1 53 (FASM Re-Assemble)

db "sqr", 0, 0, 0, 0, 0
dw FUNC66_END - FUNC66_START

FUNC66_START:
pop    ebx
cmp    ebx,0x1
jle    FUNC66_2
mov    ecx,ebx
shr    ebx,1
mov    eax,0x2
FUNC66_1:
cmp    ebx,eax
jle    FUNC66_2
mov    eax,ecx
cdq
div    ebx
add    ebx,eax
shr    ebx,1
jmp    FUNC66_1
FUNC66_2:
push   ebx 
FUNC66_END:

; =============================================================================
; Syntax:      rnd(a,b)
; Description: Random number between a and b, inclusive
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 82 bytes)
;              3.1.0 "sub r,r" -> "xor r,r"
; =============================================================================
; 8D 9F 94 00 00 00 66 8B 43 02 66 40 66 3D 38 00 75 04 66 B8 01 00 66 89 43 02 66 8B 03 66 40 66 3D 38 00 75 04 66 B8 01 00 66 89 03 0F B7 03 8D 4C 83 04 8B 01 0F B6 53 02 2B 44 93 04 89 01 5E 59 2B F1 79 04 33 C0 EB 08 46 2B D2 F7 F6 8D 04 11 50 (3.0.4 Original ASM)
; 8D 9F 94 00 00 00 66 8B 43 02 66 40 66 83 F8 38 75 04 66 B8 01 00 66 89 43 02 66 8B 03 66 40 66 83 F8 38 75 04 66 B8 01 00 66 89 03 0F B7 03 8D 4C 83 04 8B 01 0F B6 53 02 2B 44 93 04 89 01 5E 59 29 CE 79 04 31 C0 EB 08 46 29 D2 F7 F6 8D 04 11 50 (FASM Re-Assemble)
; 8D 9F 94 00 00 00 66 8B 43 02 66 40 66 83 F8 38 75 04 66 B8 01 00 66 89 43 02 66 8B 03 66 40 66 83 F8 38 75 04 66 B8 01 00 66 89 03 0F B7 03 8D 4C 83 04 8B 01 0F B6 53 02 2B 44 93 04 89 01 5E 59 29 CE 79 04 31 C0 EB 08 46 31 D2 F7 F6 8D 04 11 50 (Sub to XOR)

db "rnd", 0, 0, 0, 0, 0
dw FUNC67_END - FUNC67_START

FUNC67_START:
lea    ebx,DWORD [edi+0x94]
mov    ax,WORD [ebx+0x2]
inc    ax
cmp    ax,0x38
jne    FUNC67_1
mov    ax,0x1
FUNC67_1:
mov    WORD [ebx+0x2],ax
mov    ax,WORD [ebx]
inc    ax
cmp    ax,0x38
jne    FUNC67_2
mov    ax,0x1
FUNC67_2:
mov    WORD [ebx],ax
movzx  eax,WORD [ebx]
lea    ecx,DWORD [ebx+eax*4+0x4]
mov    eax,DWORD [ecx]
movzx  edx,BYTE [ebx+0x2]
sub    eax,DWORD [ebx+edx*4+0x4]
mov    DWORD [ecx],eax
pop    esi
pop    ecx
sub    esi,ecx
jns    FUNC67_3
xor    eax,eax
jmp    FUNC67_4
FUNC67_3:
inc    esi
xor    edx,edx
div    esi
lea    eax,DWORD [ecx+edx*1]
FUNC67_4:
push   eax 
FUNC67_END:

; =============================================================================
; Syntax:      rst(i)
; Description: UNDOCUMENTED! Sets a seed for rnd(). Parameter i has the range of 0..32767
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 10 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 9 bytes)
; =============================================================================
; 58 89 87 78 45 00 00 2B C0 50 (3.0.4 Original ASM)
; 58 89 87 78 45 00 00 29 C0 50 (FASM Re-Assemble)
; 58 89 87 78 45 00 00 6A 00 (ASM Size Optimization)

db "rst", 0, 0, 0, 0, 0
dw FUNC68_END - FUNC68_START

FUNC68_START:
pop    eax
mov    DWORD [edi+0x4578],eax
push   0x0 
FUNC68_END:

; =============================================================================
; Syntax:      ctl(i)
; Description: Value of slider i, where i is an integer between 0 and 7, inclusive.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 17 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 15 bytes)
;              3.1.0 "sub r,r" -> "xor r,r"
; =============================================================================
; 5B 2B C0 0B DB 7C 09 83 FB 07 7F 04 8B 44 9F 64 50 (3.0.4 Original ASM)
; 5B 29 C0 09 DB 7C 09 83 FB 07 7F 04 8B 44 9F 64 50 (FASM Re-Assemble)
; 58 29 DB A9 F8 FF FF FF 75 04 8B 5C 87 64 53 (ASM Size Optimization)
; 58 31 DB A9 F8 FF FF FF 75 04 8B 5C 87 64 53 (Sub to XOR)

db "ctl", 0, 0, 0, 0, 0
dw FUNC69_END - FUNC69_START

FUNC69_START:
pop    eax  ; param_i
xor    ebx,ebx
test   eax,0xfffffff8
jnz    FUNC69_DONE
mov    ebx,[edi+eax*4+0x64] ; ctl
FUNC69_DONE:
push ebx
FUNC69_END:

; =============================================================================
; Syntax:      val(i,a,b)
; Description: Value of slider i, mapped onto the range a to b
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 34 bytes)
;              3.1.0 Daniel Marschall: ASM Size Optimization (size = 32 bytes)
;              3.1.0 "sub r,r" -> "xor r,r"
;              3.1.1 fix of bug introduced in 3.1.0 (size = 33 bytes)
; =============================================================================
; 59 5E 5B 2B C0 0B DB 7C 18 83 FB 07 7F 13 8B 44 9F 64 2B CE 0F AF C1 B9 FF 00 00 00 99 F7 F9 03 C6 50 (3.0.4 Original ASM)
; 59 5E 5B 29 C0 09 DB 7C 18 83 FB 07 7F 13 8B 44 9F 64 29 F1 0F AF C1 B9 FF 00 00 00 99 F7 F9 01 F0 50 (FASM Re-Assemble)
; 59 5E 58 29 DB A9 F8 FF FF FF       75 13 8B 5C 87 64 2B CE 0F AF D9 B9 FF 00 00 00 99 F7 F9 01 F3 53 (ASM Size Optimization)
; 59 5E 58 29 DB A9 F8 FF FF FF       75 13 8B 5C 87 64 29 F1 0F AF D9 B9 FF 00 00 00 99 F7 F9 01 F3 53 (ASM Size Optimization FASM)
; 59 5E 58 31 DB A9 F8 FF FF FF       75 13 8B 5C 87 64 29 F1 0F AF D9 B9 FF 00 00 00 99 F7 F9 01 F3 53 (Sub to XOR)
; 59 5E 5B 31 C0 F7 C3 F8 FF FF FF    75 13 8B 44 9F 64 29 F1 0F AF C1 B9 FF 00 00 00 99 F7 F9 01 F0 50 (3.1.1 fix of bug introduced in 3.1.0)

db "val", 0, 0, 0, 0, 0
dw FUNC70_END - FUNC70_START

FUNC70_START:
pop    ecx  ; param_b
pop    esi  ; param_a
pop    ebx  ; param_i
xor    eax,eax
test   ebx,0xfffffff8
jnz    FUNC70_DONE
mov    eax,[edi+ebx*4+0x64] ; ctl
sub    ecx,esi
imul   eax,ecx
mov    ecx,0xff
cdq
idiv   ecx
add    eax,esi
FUNC70_DONE:
push   eax
FUNC70_END:

; =============================================================================
; Syntax:      map(i,n)
; Description: Item n from mapping table i, where i is an integer between 0 and 3, inclusive, and n is and integer between 0 and 255, inclusive
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 48 bytes)
;              3.1.0 Daniel Marschall: ASM Size optimization (size = 46 bytes)
;              3.1.0 "sub r,r" -> "xor r,r"
; =============================================================================
; 58 5B 0B DB 7C 27 83 FB 03 7F 22 0B C0 7D 04 2B C0 EB 0B B9 FF 00 00 00 3B C1 7E 02 8B C1 C1 E3 08 03 DF 0F B6 84 18 78 01 00 00 EB 02 2B C0 50 (3.0.4 Original ASM)
; 58 5B 09 DB 7C 27 83 FB 03 7F 22 09 C0 7D 04 29 C0 EB 0B B9 FF 00 00 00 39 C8 7E 02 89 C8 C1 E3 08 01 FB 0F B6 84 18 78 01 00 00 EB 02 29 C0 50 (FASM Re-Assemble)
; 5B 58 A9 FC FF FF FF       75 22 09 DB 7D 04 29 DB EB 0B B9 FF 00 00 00 39 CB 7E 02 89 CB C1 E0 08 01 F8 0F B6 9C 18 78 01 00 00 EB 02 29 DB 53 (ASM Size Optimization)
; 5B 58 A9 FC FF FF FF       75 22 09 DB 7D 04 29 DB EB 0B B9 FF 00 00 00 39 CB 7E 02 89 CB C1 E0 08 01 F8 0F B6 9C 03 78 01 00 00 EB 02 29 DB 53 (ASM Size Optimization FASM)
; 5B 58 A9 FC FF FF FF       75 22 09 DB 7D 04 31 DB EB 0B B9 FF 00 00 00 39 CB 7E 02 89 CB C1 E0 08 01 F8 0F B6 9C 03 78 01 00 00 EB 02 31 DB 53 (Sub to XOR)

db "map", 0, 0, 0, 0, 0
dw FUNC71_END - FUNC71_START

; ASM Size Anmerkung:
; 3.1.0: Optimize 0..3 check with 0x03 bitmask, eax<->ebx swapped to save another byte
; Den test von n (ebx) kann man nicht mit "test" optimieren, da dann die Semantik anders wre.
; Es soll nicht 0 zurckgegeben werden, sondern ebx soll angepasst werden:
; if n < 0   then n := 0;
; if n > 255 then n := 255;

FUNC71_START:
pop    ebx  ; param_n
pop    eax  ; param_i
test   eax,0xfffffffc
jnz FUNC71_3
or     ebx,ebx
jnl    FUNC71_1
xor    ebx,ebx
jmp    FUNC71_2
FUNC71_1:
mov    ecx,0xff
cmp    ebx,ecx
jle    FUNC71_2
mov    ebx,ecx
FUNC71_2:
shl    eax,0x8
add    eax,edi
movzx  ebx,BYTE [ebx+eax+0x0178]  ; map
jmp    FUNC71_4
FUNC71_3:
xor    ebx,ebx
FUNC71_4:
push   ebx
FUNC71_END:

; =============================================================================
; Syntax:      src(x,y,z)
; Description: Channel z for the pixel at coordinates x,y
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 94 bytes)
;              3.1.0 "sub r,r" -> "xor r,r"
; =============================================================================
; 5A 59 5B 3B 57 5C 7C 53 3B 57 60 7D 4E 8B 77 44 3B DE 7D 02 8B DE 8B 47 48 3B D8 7C 03 8D 58 FF 8B 47 50 3B C8 7D 02 8B C8 8B 47 54 3B C8 7C 03 8D 48 FF 8B 87 A8 45 00 00 2B 57 5C 03 C2 2B 4F 50 0F AF 8F A0 45 00 00 03 C1 2B DE 8B 57 60 2B 57 5C 0F AF DA 0F B6 04 18 EB 02 2B C0 50 (3.0.4 Original ASM)
; 5A 59 5B 3B 57 5C 7C 53 3B 57 60 7D 4E 8B 77 44 39 F3 7D 02 89 F3 8B 47 48 39 C3 7C 03 8D 58 FF 8B 47 50 39 C1 7D 02 89 C1 8B 47 54 39 C1 7C 03 8D 48 FF 8B 87 A8 45 00 00 2B 57 5C 01 D0 2B 4F 50 0F AF 8F A0 45 00 00 01 C8 29 F3 8B 57 60 2B 57 5C 0F AF DA 0F B6 04 18 EB 02 29 C0 50 (FASM Re-Assemble)
; 5A 59 5B 3B 57 5C 7C 53 3B 57 60 7D 4E 8B 77 44 39 F3 7D 02 89 F3 8B 47 48 39 C3 7C 03 8D 58 FF 8B 47 50 39 C1 7D 02 89 C1 8B 47 54 39 C1 7C 03 8D 48 FF 8B 87 A8 45 00 00 2B 57 5C 01 D0 2B 4F 50 0F AF 8F A0 45 00 00 01 C8 29 F3 8B 57 60 2B 57 5C 0F AF DA 0F B6 04 18 EB 02 31 C0 50 (Sub to XOR)

db "src", 0, 0, 0, 0, 0
dw FUNC72_END - FUNC72_START

FUNC72_START:
pop    edx
pop    ecx
pop    ebx
cmp    edx,DWORD [edi+0x5c]
jl     FUNC72_5
cmp    edx,DWORD [edi+0x60]
jge    FUNC72_5
mov    esi,DWORD [edi+0x44]
cmp    ebx,esi
jge    FUNC72_1
mov    ebx,esi
FUNC72_1:
mov    eax,DWORD [edi+0x48]
cmp    ebx,eax
jl     FUNC72_2
lea    ebx,DWORD [eax-0x1]
FUNC72_2:
mov    eax,DWORD [edi+0x50]
cmp    ecx,eax
jge    FUNC72_3
mov    ecx,eax
FUNC72_3:
mov    eax,DWORD [edi+0x54]
cmp    ecx,eax
jl     FUNC72_4
lea    ecx,DWORD [eax-0x1]
FUNC72_4:
mov    eax,DWORD [edi+0x45a8]
sub    edx,DWORD [edi+0x5c]
add    eax,edx
sub    ecx,DWORD [edi+0x50]
imul   ecx,DWORD [edi+0x45a0]
add    eax,ecx
sub    ebx,esi
mov    edx,DWORD [edi+0x60]
sub    edx,DWORD [edi+0x5c]
imul   ebx,edx
movzx  eax,BYTE [eax+ebx*1]
jmp    FUNC72_6
FUNC72_5:
xor    eax,eax
FUNC72_6:
push   eax 
FUNC72_END:

; =============================================================================
; Syntax:      cnv(m11,m12,m13,m21,m22,m23,m31,m32,m33,d)
; Description: "Convolve". Applies a convolution matrix and divides with d.
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 187 bytes)
;              3.1.0 Daniel Marschall: Fixed out-of-bounds crash (at borders). Now using neighbor pixels as off-canvas values (cnv variant 3; size = 347 bytes).
; =============================================================================
; 8B 04 24 0B C0 0F 84 AC 00 00 00 8B 8F A8 45 00 00 8B 47 58 2B 47 5C 03 C8 8B 47 4C 48 2B 47 50 F7 A7 A0 45 00 00 03 C8 8B 47 40 48 2B 47 44 8B 5F 60 2B 5F 5C F7 E3 03 C8 2B C0 8B D1 0F B6 32 0F AF 74 24 24 03 C6 03 D3 0F B6 32 0F AF 74 24 20 03 C6 03 D3 0F B6 32 0F AF 74 24 1C 03 C6 03 8F A0 45 00 00 8B D1 0F B6 32 0F AF 74 24 18 03 C6 03 D3 0F B6 32 0F AF 74 24 14 03 C6 03 D3 0F B6 32 0F AF 74 24 10 03 C6 03 8F A0 45 00 00 8B D1 0F B6 32 0F AF 74 24 0C 03 C6 03 D3 0F B6 32 0F AF 74 24 08 03 C6 03 D3 0F B6 32 0F AF 74 24 04 03 C6 99 F7 3C 24 83 C4 28 50 (3.0.4 Original ASM)
; 8B 04 24 09 C0 0F 84 AC 00 00 00 8B 8F A8 45 00 00 8B 47 58 2B 47 5C 01 C1 8B 47 4C 48 2B 47 50 F7 A7 A0 45 00 00 01 C1 8B 47 40 48 2B 47 44 8B 5F 60 2B 5F 5C F7 E3 01 C1 29 C0 89 CA 0F B6 32 0F AF 74 24 24 01 F0 01 DA 0F B6 32 0F AF 74 24 20 01 F0 01 DA 0F B6 32 0F AF 74 24 1C 01 F0 03 8F A0 45 00 00 89 CA 0F B6 32 0F AF 74 24 18 01 F0 01 DA 0F B6 32 0F AF 74 24 14 01 F0 01 DA 0F B6 32 0F AF 74 24 10 01 F0 03 8F A0 45 00 00 89 CA 0F B6 32 0F AF 74 24 0C 01 F0 01 DA 0F B6 32 0F AF 74 24 08 01 F0 01 DA 0F B6 32 0F AF 74 24 04 01 F0 99 F7 3C 24 83 C4 28 50 (FASM Re-Assemble)
; 8B 04 24 09 C0 0F 84 4C 01 00 00 8B 8F A8 45 00 00 8B 47 58 2B 47 5C 01 C1 8B 47 4C 48 2B 47 50 F7 A7 A0 45 00 00 01 C1 8B 47 40 48 2B 47 44 8B 5F 60 2B 5F 5C F7 E3 01 C1 31 C0 89 CA 52 8B 77 4C 09 F6 7F 06 03 97 A0 45 00 00 8B 77 40 09 F6 7F 02 01 DA 0F B6 32 5A 0F AF 74 24 24 01 F0 01 DA 52 8B 77 4C 09 F6 7F 06 03 97 A0 45 00 00 0F B6 32 5A 0F AF 74 24 20 01 F0 01 DA 52 8B 77 4C 09 F6 7F 06 03 97 A0 45 00 00 8B 77 48 4E 3B 77 40 7F 02 29 DA 0F B6 32 5A 0F AF 74 24 1C 01 F0 03 8F A0 45 00 00 89 CA 52 8B 77 40 09 F6 7F 02 01 DA 0F B6 32 5A 0F AF 74 24 18 01 F0 01 DA 0F B6 32 0F AF 74 24 14 01 F0 01 DA 52 8B 77 48 4E 3B 77 40 7F 02 29 DA 0F B6 32 5A 0F AF 74 24 10 01 F0 03 8F A0 45 00 00 89 CA 52 8B 77 54 4E 3B 77 4C 7F 06 2B 97 A0 45 00 00 8B 77 40 09 F6 7F 02 01 DA 0F B6 32 5A 0F AF 74 24 0C 01 F0 01 DA 52 8B 77 54 4E 3B 77 4C 7F 06 2B 97 A0 45 00 00 0F B6 32 5A 0F AF 74 24 08 01 F0 01 DA 52 8B 77 54 4E 3B 77 4C 7F 06 2B 97 A0 45 00 00 8B 77 48 4E 3B 77 40 7F 02 29 DA 0F B6 32 5A 0F AF 74 24 04 01 F0 99 F7 3C 24 83 C4 28 50 (out-of-bounds fix)

db "cnv", 0, 0, 0, 0, 0
dw FUNC73_END - FUNC73_START

FUNC73_START:
mov eax,[esp] ; param_d
or eax,eax
jz FUNC73_div_zero
mov ecx,[edi+0x000045a8] ;CANVAS_OFFSET
mov eax,[edi+0x58] ;z
sub eax,[edi+0x5c] ;zmin
add ecx,eax
mov eax,[edi+0x4c] ;y
dec eax
sub eax,[edi+0x50] ;ymin
mul dword [edi+0x000045a0] ;ROW_SIZE
add ecx,eax
mov eax,[edi+0x40] ;x
dec eax
sub eax,[edi+0x44] ;xmin
mov ebx,[edi+0x60] ;Z
sub ebx,[edi+0x5c] ;zmin
mul ebx
add ecx,eax
xor eax,eax ; Optimized for better performance. Was "sub eax,eax"
mov edx,ecx ; begin new line (-> f11)

; Field f11
push edx ; backup position
mov esi,[edi+0x4c] ;y
or esi,esi
jnle FUNC73_f11_y_ok ; if (y>0) jmp FUNC73_f11_y_ok
        ; y=0
        ; Correct y value.
        add edx,[edi+0x000045a0] ;ROW_SIZE ; Next line (-> f21)
FUNC73_f11_y_ok:
        mov esi,[edi+0x40] ;x
        or esi,esi
        jnle FUNC73_f11_yx_ok ; if (x>0) jmp FUNC73_f11_yx_ok

	; x=0
        ; Correct x value.
	add edx,ebx ; Next pixel (-> f22 if y=0, else f12)
FUNC73_f11_yx_ok:
movzx esi,BYTE [edx]
pop edx ; restore position
imul esi,[esp+0x24] ;param_m11
add eax,esi
add edx,ebx ; Next pixel (-> f12)

; Field f12
push edx ; backup position
mov esi,dword [edi+0x4c] ;y
or esi,esi
jnle FUNC73_f12_yx_ok ; if (y>0) jmp FUNC73_f12_yx_ok
        ; y=0
        ; Correct y value.
        add edx,dword [edi+0x000045a0] ;ROW_SIZE ; Next line (-> f22)
FUNC73_f12_yx_ok:
movzx esi,BYTE [edx]
pop edx ; restore position
imul esi,[esp+0x20] ;param_m12
add eax,esi
add edx,ebx ; Next pixel (-> f13)

; Field f13
push edx ; backup position
mov esi,dword [edi+0x4c] ;y
or esi,esi
jnle FUNC73_f13_y_ok ; if (y>0) jmp FUNC73_f13_y_ok
        ; y=0
        ; Correct y value.
        add edx,dword [edi+0x000045a0] ;ROW_SIZE ; Next line (-> f23)
FUNC73_f13_y_ok:
        mov esi,dword [edi+0x48] ;X
        dec esi
        cmp esi,dword [edi+0x40] ;x
        jnle FUNC73_f13_yx_ok ; if (x<X-1) jmp FUNC73_f13_yx_ok

	; x=X-1
        ; Correct x value.
	sub edx,ebx ; Prev pixel (-> f22 if y=0, else f12)
FUNC73_f13_yx_ok:
movzx esi,BYTE [edx]
pop edx ; restore position
imul esi,[esp+0x1c] ;param_m13
add eax,esi
add ecx,dword [edi+0x000045a0]	;ROW_SIZE ; begin new line
mov edx,ecx ; Next line (-> f21)

; Field f21 
push edx ; backup position
mov esi,dword [edi+0x40] ;x
or esi,esi
jnle FUNC73_f21_yx_ok ; if (x>0) jmp FUNC73_f21_yx_ok
	; x=0
        ; Correct x value.
	add edx,ebx ; Next pixel (-> f22)
FUNC73_f21_yx_ok:
movzx esi,BYTE [edx]
pop edx ; restore position
imul esi,[esp+0x18] ;param_m21
add eax,esi
add edx,ebx ; Next pixel (-> f22)

; Field f22 
movzx esi,BYTE [edx]
imul esi,[esp+0x14] ;param_m22
add eax,esi
add edx,ebx ; Next pixel (-> f23)

; Field f23 
push edx ; backup position
mov esi,dword [edi+0x48] ;X
dec esi
cmp esi,dword [edi+0x40] ;x
jnle FUNC73_f23_yx_ok ; if (x<X-1) jmp FUNC73_f23_yx_ok
	; x=X-1
        ; Correct x value.
	sub edx,ebx ; Prev pixel (-> f22)
FUNC73_f23_yx_ok:
movzx esi,BYTE [edx]
pop edx ; restore position
imul esi,[esp+0x10] ;param_m23
add eax,esi
add ecx,dword [edi+0x000045a0]	;ROW_SIZE ; begin new line
mov edx,ecx ; Next line (-> f31)

; Field f31 
push edx ; backup position
mov esi,dword [edi+0x54] ;Y
dec esi
cmp esi,dword [edi+0x4c] ;y
jnle FUNC73_f31_y_ok ; if (y<Y-1) jmp FUNC73_f31_y_ok
        ; y=Y-1
        ; Correct y value.
        sub edx,dword [edi+0x000045a0] ;ROW_SIZE ; Prev line (-> f21)
FUNC73_f31_y_ok:
        mov esi,dword [edi+0x40] ;x
        or esi,esi
        jnle FUNC73_f31_yx_ok ; if (x>0) jmp FUNC73_f31_yx_ok

	; x=0
        ; Correct x value.
	add edx,ebx ; Next pixel (-> f22 if y=Y-1, else f32)
FUNC73_f31_yx_ok:
movzx esi,BYTE [edx]
pop edx ; restore position
imul esi,[esp+0x0c] ;param_m31
add eax,esi
add edx,ebx ; Next pixel (-> f32)

; Field f32 
push edx ; backup position
mov esi,dword [edi+0x54] ;Y
dec esi
cmp esi,dword [edi+0x4c] ;y
jnle FUNC73_f32_yx_ok ; if (y<Y-1) jmp FUNC73_f32_yx_ok
	; y=Y-1
        ; Correct y value.
        sub edx,dword [edi+0x000045a0] ;ROW_SIZE ; Prev line (-> f22)
FUNC73_f32_yx_ok:
movzx esi,BYTE [edx]
pop edx ; restore position
imul esi,[esp+0x08] ;param_m32
add eax,esi
add edx,ebx ; Next pixel (-> f33)

; Field f33 
push edx ; backup position
mov esi,dword [edi+0x54] ;Y
dec esi
cmp esi,dword [edi+0x4c] ;y
jnle FUNC73_f33_y_ok ; if (y<Y-1) jmp FUNC73_f33_y_ok
	; y=Y-1
        ; Correct y value.
        sub edx,dword [edi+0x000045a0] ;ROW_SIZE ; Prev line (-> f23)
FUNC73_f33_y_ok:
        mov esi,dword [edi+0x48] ;X
        dec esi
        cmp esi,dword [edi+0x40] ;x
        jnle FUNC73_f33_yx_ok ; if (x<X-1) jmp FUNC73_f33_yx_ok

	; x=X-1
        ; Correct x value.
	sub edx,ebx ; Prev pixel (-> f22 if y=Y-1, else f32)
FUNC73_f33_yx_ok:
movzx esi,BYTE [edx]
pop edx ; restore position
imul esi,[esp+0x04] ;param_m33
add eax,esi

; Epilog 
cdq
idiv dword [esp] ;param_d
FUNC73_div_zero:
add esp,0x28 ; = 10x pop
push eax

FUNC73_END:

; =============================================================================
; Syntax:      rad(d,m,z)
; Description: Channel z in the source image, which is m units away, at an angle of d, from the center of the image
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 158 bytes)
;              3.1.0 "sub r,r" -> "xor r,r"
; =============================================================================
; 5E 8B 04 24 8B 5C 24 04 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 8B 4F 48 0F AC D0 0E 2B 4F 44 D1 E9 03 C8 0B C9 7D 04 2B C9 EB 0A 8B 47 48 3B C8 7C 03 8D 48 FF 58 5B 81 EB 00 01 00 00 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 8B 5F 54 0F AC D0 0E 2B 5F 50 D1 EB 03 D8 0B DB 7D 04 2B DB EB 0A 8B 47 54 3B D8 7C 03 8D 58 FF 8B 57 60 2B 57 5C 03 B7 A8 45 00 00 0F AF 9F A0 45 00 00 0F AF CA 33 C0 03 D9 8A 04 1E 50 (3.0.4 Original ASM)
; 5E 8B 04 24 8B 5C 24 04 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 8B 4F 48 0F AC D0 0E 2B 4F 44 D1 E9 01 C1 09 C9 7D 04 29 C9 EB 0A 8B 47 48 39 C1 7C 03 8D 48 FF 58 5B 81 EB 00 01 00 00 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 8B 5F 54 0F AC D0 0E 2B 5F 50 D1 EB 01 C3 09 DB 7D 04 29 DB EB 0A 8B 47 54 39 C3 7C 03 8D 58 FF 8B 57 60 2B 57 5C 03 B7 A8 45 00 00 0F AF 9F A0 45 00 00 0F AF CA 31 C0 01 CB 8A 04 1E 50 (FASM Re-Assemble)
; 5E 8B 04 24 8B 5C 24 04 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 8B 4F 48 0F AC D0 0E 2B 4F 44 D1 E9 01 C1 09 C9 7D 04 31 C9 EB 0A 8B 47 48 39 C1 7C 03 8D 48 FF 58 5B 81 EB 00 01 00 00 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 8B 5F 54 0F AC D0 0E 2B 5F 50 D1 EB 01 C3 09 DB 7D 04 31 DB EB 0A 8B 47 54 39 C3 7C 03 8D 58 FF 8B 57 60 2B 57 5C 03 B7 A8 45 00 00 0F AF 9F A0 45 00 00 0F AF CA 31 C0 01 CB 8A 04 1E 50 (Sub to XOR)

db "rad", 0, 0, 0, 0, 0
dw FUNC74_END - FUNC74_START

FUNC74_START:
pop    esi
mov    eax,DWORD [esp]
mov    ebx,DWORD [esp+0x4]
and    ebx,0x3ff
movsx  ebx,WORD [edi+ebx*2+0x578]
imul   ebx
add    eax,0x1fff
adc    edx,0x0
mov    ecx,DWORD [edi+0x48]
shrd   eax,edx,0xe
sub    ecx,DWORD [edi+0x44]
shr    ecx,1
add    ecx,eax
or     ecx,ecx
jge    FUNC74_1
xor    ecx,ecx
jmp    FUNC74_2
FUNC74_1:
mov    eax,DWORD [edi+0x48]
cmp    ecx,eax
jl     FUNC74_2
lea    ecx,DWORD [eax-0x1]
FUNC74_2:
pop    eax  ; param m
pop    ebx  ; param d
sub    ebx,0x100
and    ebx,0x3ff
movsx  ebx,WORD [edi+ebx*2+0x578]
imul   ebx
add    eax,0x1fff
adc    edx,0x0
mov    ebx,DWORD [edi+0x54]
shrd   eax,edx,0xe
sub    ebx,DWORD [edi+0x50]
shr    ebx,1
add    ebx,eax
or     ebx,ebx
jge    FUNC74_3
xor    ebx,ebx
jmp    FUNC74_4
FUNC74_3:
mov    eax,DWORD [edi+0x54]  ; Y
cmp    ebx,eax
jl     FUNC74_4
lea    ebx,DWORD [eax-0x1]
FUNC74_4:
mov    edx,DWORD [edi+0x60]
sub    edx,DWORD [edi+0x5c]
add    esi,DWORD [edi+0x45a8]
imul   ebx,DWORD [edi+0x45a0]
imul   ecx,edx
xor    eax,eax
add    ebx,ecx
mov    al,BYTE  [esi+ebx*1]
push   eax 
FUNC74_END:

; =============================================================================
; Syntax:      c2d(x,y)
; Description: Angle displacement of the pixel at coordinates x,y
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 97 bytes)
; =============================================================================
; 58 5B 33 C9 0B C0 7D 05 F7 D8 80 C9 04 0B DB 7D 05 F7 DB 80 C9 03 3B C3 7E 04 80 F1 01 93 0B C0 7E 1B C1 E0 0A 99 F7 F3 74 13 66 8B 84 47 76 0D 00 00 C1 E0 09 99 BB 40 24 03 00 F7 FB 66 0F BA E1 00 73 07 F7 D8 05 00 01 00 00 66 0F BA E1 01 73 05 05 00 01 00 00 66 0F BA E1 02 73 02 F7 D8 50 (3.0.4 Original ASM)
; 58 5B 31 C9 09 C0 7D 05 F7 D8 80 C9 04 09 DB 7D 05 F7 DB 80 C9 03 39 D8 7E 04 80 F1 01 93 09 C0 7E 1B C1 E0 0A 99 F7 F3 74 13 66 8B 84 47 76 0D 00 00 C1 E0 09 99 BB 40 24 03 00 F7 FB 66 0F BA E1 00 73 07 F7 D8 05 00 01 00 00 66 0F BA E1 01 73 05 05 00 01 00 00 66 0F BA E1 02 73 02 F7 D8 50 (FASM Re-Assemble)

db "c2d", 0, 0, 0, 0, 0
dw FUNC75_END - FUNC75_START

FUNC75_START:
pop    eax
pop    ebx
xor    ecx,ecx
or     eax,eax
jge    FUNC75_1
neg    eax
or     cl,0x4
FUNC75_1:
or     ebx,ebx
jge    FUNC75_2
neg    ebx
or     cl,0x3
FUNC75_2:
cmp    eax,ebx
jle    FUNC75_3
xor    cl,0x1
xchg   ebx,eax
FUNC75_3:
or     eax,eax
jle    FUNC75_4
shl    eax,0xa
cdq
div    ebx
je     FUNC75_4
mov    ax,WORD [edi+eax*2+0xd76]
shl    eax,0x9
cdq
mov    ebx,0x32440
idiv   ebx
FUNC75_4:
bt     cx,0x0
jae    FUNC75_5
neg    eax
add    eax,0x100
FUNC75_5:
bt     cx,0x1
jae    FUNC75_6
add    eax,0x100
FUNC75_6:
bt     cx,0x2
jae    FUNC75_7
neg    eax
FUNC75_7:
push   eax 
FUNC75_END:

; =============================================================================
; Syntax:      c2m(x,y)
; Description: Magnitude displacement of the pixel at coordinates x,y
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 57 bytes)
; =============================================================================
; 5B 58 8B D0 C1 FA 1F 33 C2 2B C2 8B D3 C1 FA 1F 33 DA 2B DA 3B C3 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 03 C3 50 (3.0.4 Original ASM)
; 5B 58 89 C2 C1 FA 1F 31 D0 29 D0 89 DA C1 FA 1F 31 D3 29 D3 39 D8 75 07 B8 0A 6A 00 00 EB 11 7E 01 93 C1 E0 0A 99 F7 F3 0F B7 84 47 78 15 00 00 F7 E3 0F AC D0 10 01 D8 50 (FASM Re-Assemble)

db "c2m", 0, 0, 0, 0, 0
dw FUNC76_END - FUNC76_START

FUNC76_START:
pop    ebx
pop    eax
mov    edx,eax
sar    edx,0x1f
xor    eax,edx
sub    eax,edx
mov    edx,ebx
sar    edx,0x1f
xor    ebx,edx
sub    ebx,edx
cmp    eax,ebx
jne    FUNC76_1
mov    eax,0x6a0a
jmp    FUNC76_3
FUNC76_1:
jle    FUNC76_2
xchg   ebx,eax
FUNC76_2:
shl    eax,0xa
cdq
div    ebx
movzx  eax,WORD [edi+eax*2+0x1578]
FUNC76_3:
mul    ebx
shrd   eax,edx,0x10
add    eax,ebx
push   eax 
FUNC76_END:

; =============================================================================
; Syntax:      r2x(d,m)
; Description: x displacement of the pixel m units away, at an angle of d, from an arbitrary center
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 31 bytes)
; =============================================================================
; 58 5B 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 0F AC D0 0E 50 (3.0.4 Original ASM)
; 58 5B 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 0F AC D0 0E 50 (FASM Re-Assemble)

db "r2x", 0, 0, 0, 0, 0
dw FUNC77_END - FUNC77_START

FUNC77_START:
pop    eax
pop    ebx
and    ebx,0x3ff
movsx  ebx,WORD [edi+ebx*2+0x578]
imul   ebx
add    eax,0x1fff
adc    edx,0x0
shrd   eax,edx,0xe
push   eax 
FUNC77_END:

; =============================================================================
; Syntax:      r2y(d,m)
; Description: y displacement of the pixel m units away, at an angle of d, from an arbitrary center
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 37 bytes)
; =============================================================================
; 58 5B 81 EB 00 01 00 00 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 0F AC D0 0E 50 (3.0.4 Original ASM)
; 58 5B 81 EB 00 01 00 00 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 F7 EB 05 FF 1F 00 00 83 D2 00 0F AC D0 0E 50 (FASM Re-Assemble)

db "r2y", 0, 0, 0, 0, 0
dw FUNC78_END - FUNC78_START

FUNC78_START:
pop    eax
pop    ebx
sub    ebx,0x100
and    ebx,0x3ff
movsx  ebx,WORD [edi+ebx*2+0x578]
imul   ebx
add    eax,0x1fff
adc    edx,0x0
shrd   eax,edx,0xe
push   eax 
FUNC78_END:

; =============================================================================
; Syntax:      sin(x)
; Description: Sine function of x, where x is an integer between 0 and 1024, inclusive, and the value returned is an integer between -512 and 512, inclusive (Windows) or -1024 and 1024, inclusive (Mac OS)
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 32 bytes)
; =============================================================================
; 58 2D 00 01 00 00 8B D8 C1 FB 1F 33 C3 2B C3 25 FF 03 00 00 0F BF 84 47 78 05 00 00 C1 F8 05 50 (3.0.4 Original ASM)
; 58 2D 00 01 00 00 89 C3 C1 FB 1F 31 D8 29 D8 25 FF 03 00 00 0F BF 84 47 78 05 00 00 C1 F8 05 50 (FASM Re-Assemble)

db "sin", 0, 0, 0, 0, 0
dw FUNC79_END - FUNC79_START

FUNC79_START:
pop    eax
sub    eax,0x100
mov    ebx,eax
sar    ebx,0x1f
xor    eax,ebx
sub    eax,ebx
and    eax,0x3ff
movsx  eax,WORD [edi+eax*2+0x578]
sar    eax,0x5
push   eax 
FUNC79_END:

; =============================================================================
; Syntax:      cos(x)
; Description: Cosine function of x, where x is an integer between 0 and 1024, inclusive, and the value returned is an integer between -512 and 512, inclusive (Windows) or -1024 and 1024, inclusive (Mac OS)
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 27 bytes)
; =============================================================================
; 58 8B D8 C1 FB 1F 33 C3 2B C3 25 FF 03 00 00 0F BF 84 47 78 05 00 00 C1 F8 05 50 (3.0.4 Original ASM)
; 58 89 C3 C1 FB 1F 31 D8 29 D8 25 FF 03 00 00 0F BF 84 47 78 05 00 00 C1 F8 05 50 (FASM Re-Assemble)

db "cos", 0, 0, 0, 0, 0
dw FUNC80_END - FUNC80_START

FUNC80_START:
pop    eax
mov    ebx,eax
sar    ebx,0x1f
xor    eax,ebx
sub    eax,ebx
and    eax,0x3ff
movsx  eax,WORD [edi+eax*2+0x578]
sar    eax,0x5
push   eax 
FUNC80_END:

; =============================================================================
; Syntax:      tan(x)
; Description: Bounded tangent function of x, where x is an integer between -256 and 256, inclusive, and the value returned is an integer between -512 and 512, inclusive (Windows) or -1024 and 1024, inclusive (Mac OS)
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 60 bytes)
; =============================================================================
; 58 8B D8 0B DB 7D 02 F7 DB 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 2D 00 01 00 00 79 02 F7 D8 25 FF 03 00 00 0F BF 84 47 78 05 00 00 0B DB 75 04 33 C0 EB 06 C1 E0 0A 99 F7 FB 50 (3.0.4 Original ASM)
; 58 89 C3 09 DB 7D 02 F7 DB 81 E3 FF 03 00 00 0F BF 9C 5F 78 05 00 00 2D 00 01 00 00 79 02 F7 D8 25 FF 03 00 00 0F BF 84 47 78 05 00 00 09 DB 75 04 31 C0 EB 06 C1 E0 0A 99 F7 FB 50 (FASM Re-Assemble)

db "tan", 0, 0, 0, 0, 0
dw FUNC81_END - FUNC81_START

FUNC81_START:
pop    eax
mov    ebx,eax
or     ebx,ebx
jge    FUNC81_1
neg    ebx
FUNC81_1:
and    ebx,0x3ff
movsx  ebx,WORD [edi+ebx*2+0x578]
sub    eax,0x100
jns    FUNC81_2
neg    eax
FUNC81_2:
and    eax,0x3ff
movsx  eax,WORD [edi+eax*2+0x578]
or     ebx,ebx
jne    FUNC81_3
xor    eax,eax
jmp    FUNC81_4
FUNC81_3:
shl    eax,0xa
cdq
idiv   ebx
FUNC81_4:
push   eax 
FUNC81_END:

; =============================================================================
; Syntax:      get(i)
; Description: Returns the current cell value at i
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 20 bytes)
;              3.1.0 Daniel Marschall: Optimize size (size = 16 bytes)
; =============================================================================
; 58 0B C0 7C 0E 3D FF 00 00 00 7F 07 8B 84 87 78 35 00 00 50 (3.0.4 Original ASM)
; 58 09 C0 7C 0E 3D FF 00 00 00 7F 07 8B 84 87 78 35 00 00 50 (FASM Re-Assemble)
; 58 A9 00 FF FF FF 75 07 8B 84 87 78 35 00 00 50 (ASM Size Optimization)

db "get", 0, 0, 0, 0, 0
dw FUNC82_END - FUNC82_START

FUNC82_START:
pop    eax  ; param i
test   eax,0xffffff00  ; 3.1.0: 0..255 check with 0xFF bitmask
jnz    FUNC82_DONE
mov    eax,DWORD [edi+eax*4+0x3578]
FUNC82_DONE:
push   eax 
FUNC82_END:

; =============================================================================
; Syntax:      put(v,i)
; Description: Puts the new value v into cell i
; Changelog:   3.0.4 Adobe Systems, Inc. (size = 21 bytes)
;              3.1.0 Daniel Marschall: Optimize size (size = 17 bytes)
; =============================================================================
; 58 5B 0B C0 7C 0E 3D FF 00 00 00 7F 07 89 9C 87 78 35 00 00 53 (3.0.4 Original ASM)
; 58 5B 09 C0 7C 0E 3D FF 00 00 00 7F 07 89 9C 87 78 35 00 00 53 (FASM Re-Assemble)
; 58 5B A9 00 FF FF FF 75 07 89 9C 87 78 35 00 00 53 (ASM Size Optimization)

db "put", 0, 0, 0, 0, 0
dw FUNC83_END - FUNC83_START

FUNC83_START:
pop    eax  ; param i
pop    ebx  ; param v
test   eax,0xffffff00  ; 3.1.0: 0..255 check with 0xFF bitmask
jnz    FUNC83_DONE
mov    DWORD [edi+eax*4+0x3578],ebx
FUNC83_DONE:
push   ebx 
FUNC83_END:

; =============================================================================
; Syntax:      pow(b,e)
; Description: Calculates the base to the exponent power, that is, b^e
; Changelog:   3.1.0 Daniel Marschall (size = 54 bytes)
; =============================================================================
; 5E 59 89 F0 99 31 D0 29 D0 31 D2 42 EB 11 D1 E8 89 CB 0F AF D9 89 D9 A8 01 74 F3 48 0F AF D1 85 C0 7F F4 85 F6 7D 0C 83 FA 01 74 07 83 FA FF 74 02 31 D2 52 (Original)

db "pow", 0, 0, 0, 0, 0
dw FUNC84_END - FUNC84_START

FUNC84_START:
pop esi                 ; esi = param_exp
pop ecx                 ; ecx = param_base
mov eax,esi             ; .
cdq                     ; .
xor eax,edx             ; .
sub eax,edx             ; y = abs(param_exp);
xor edx,edx             ; .
inc edx                 ; result = 1;
jmp FUNC84_3            ; goto FUNC84_3;
FUNC84_1:               ; FUNC84_1:
shr eax,1               ; y = y >> 1;
mov ebx,ecx             ; .
imul ebx,ecx            ; .
mov ecx,ebx             ; param_base = param_base * param_base;
FUNC84_2:               ; FUNC84_2:
test al,0x01             ; .
jz FUNC84_1             ; if (y & 1 == 0) goto FUNC84_1; ; not odd
dec eax                 ; y--;
imul edx,ecx            ; result = result * param_base;
FUNC84_3:               ; FUNC84_3:
test eax,eax            ; .
jnle FUNC84_2           ; if (y > 0) goto FUNC84_2;
test esi,esi            ; .
jnl FUNC84_4            ; if (param_exp >= 0) goto FUNC84_4; ; return result;
cmp edx,0x01             ; .
jz FUNC84_4             ; if (result == 1) goto FUNC84_4; ; return result;
cmp edx,-0x01            ; .
jz FUNC84_4             ; if (result == -1) goto FUNC84_4; ; return result;
xor edx,edx             ; result = 0;
FUNC84_4:               ; FUNC84_4:
push edx                ; return result;
FUNC84_END:

; =============================================================================

db 0, 0, 0, 0, 0, 0, 0, 0
dw 0
